//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Core/Computation/SpecularComputation.cpp
//! @brief     Implements class SpecularComputation.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Core/Computation/SpecularComputation.h"
#include "Base/Progress/ProgressHandler.h"
#include "Resample/Element/SpecularElement.h"
#include "Resample/Flux/IFlux.h"
#include "Resample/Processed/ProcessedSample.h"
#include "Resample/Specular/ComputeFluxMagnetic.h"
#include "Resample/Specular/ComputeFluxScalar.h"
#include "Sample/Multilayer/MultiLayer.h"

SpecularComputation::SpecularComputation(const ProcessedSample& re_sample,
                                         const SimulationOptions& options,
                                         ProgressHandler& progress, SpecularElementIter begin_it,
                                         SpecularElementIter end_it)
    : IComputation(std::move(re_sample), options, progress), m_begin_it(begin_it), m_end_it(end_it)
{
}

SpecularComputation::~SpecularComputation() = default;

void SpecularComputation::runProtected()
{
    const SliceStack& slices = m_re_sample.averageSlices();

    for (auto it = m_begin_it; it != m_end_it; ++it) {
        SpecularElement& ele = *it;
        if (!ele.isCalculated())
            continue;
        std::vector<complex_t> kz_stack = ele.produceKz(slices);
        if (m_re_sample.polarizing()) {
            const Eigen::Matrix2cd R = Compute::SpecularMagnetic::topLayerR(slices, kz_stack, true);

            const auto& polarization = ele.polarizationHandler().polarizerMatrix();
            const auto& analyzer = ele.polarizationHandler().analyzerMatrix();

            const complex_t trace = (polarization * R.adjoint() * analyzer * R).trace();

            ele.setIntensity(std::abs(trace));

        } else {
            const complex_t R = Compute::SpecularScalar::topLayerR(slices, kz_stack);
            ele.setIntensity(std::norm(R));
        }

        stepProgress();
    }
}
