//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Core/Simulation/ISimulation.h
//! @brief     Defines interface ISimulation.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef USER_API
#ifndef BORNAGAIN_CORE_SIMULATION_ISIMULATION_H
#define BORNAGAIN_CORE_SIMULATION_ISIMULATION_H

#include "Device/Instrument/Instrument.h"
#include "Param/Distrib/DistributionHandler.h"

template <class T> class OutputData;
class IBackground;
class IComputation;
class ICoordSystem;
class MultiLayer;
class ProcessedSample;
class ProgressHandler;
class SimulationOptions;
class SimulationResult;

//! Abstract base class of OffSpecularSimulation, GISASSimulation and SpecularSimulation.
//! Holds the common infrastructure to run a simulation: multithreading, batch processing,
//! weighting over parameter distributions, ...
//! @ingroup simulation

class ISimulation : public INode {
public:
    ISimulation(const Beam& beam, const MultiLayer& sample, const IDetector& detector);
    ISimulation();
    virtual ~ISimulation();

    //! Run a simulation, possibly averaged over parameter distributions
    void runSimulation();

    //! Run a simulation in a MPI environment
    void runMPISimulation();

    const Instrument& instrument() const { return m_instrument; }
    Instrument& instrument() { return m_instrument; }

    Beam& beam() { return m_instrument.beam(); }
    const Beam& beam() const { return m_instrument.beam(); }

    IDetector& detector() { return m_instrument.detector(); }
    const IDetector& detector() const { return m_instrument.detector(); }

    void setSample(const MultiLayer& sample);
    const MultiLayer* sample() const;

    void setBackground(const IBackground& bg);
    const IBackground* background() const { return m_background.get(); }

    //! Returns the total number of the intensity values in the simulation result
    virtual size_t intensityMapSize() const = 0;

    //! Returns the results of the simulation in a format that supports unit conversion and export
    //! to numpy arrays
    virtual SimulationResult result() const = 0;

    void addParameterDistribution(ParameterDistribution::WhichParameter whichParameter,
                                  const IDistribution1D& distribution, size_t nbr_samples,
                                  double sigma_factor = 0.0,
                                  const RealLimits& limits = RealLimits());
    void addParameterDistribution(const ParameterDistribution& par_distr);
    const std::vector<ParameterDistribution>& getDistributions() const;

    const SimulationOptions& options() const;
    SimulationOptions& options();

    void subscribe(const std::function<bool(size_t)>& inform);
    void setTerminalProgressMonitor();

    std::vector<const INode*> getChildren() const;

    std::string unitOfParameter(ParameterDistribution::WhichParameter which) const;

    friend class MPISimulation;

#ifndef SWIG
    virtual ICoordSystem* createCoordSystem() const = 0;
#endif // SWIG

protected:
    //! Put into a clean state for running a simulation
    virtual void prepareSimulation();

    //! Creates the appropriate data structure (e.g. 2D intensity map) from the calculated
    //! DiffuseElement objects
    virtual void transferResultsToIntensityMap() {}

    //! Initializes the vector of ISimulation elements
    virtual void initElementVector() = 0;

    virtual void updateIntensityMap() {}

    //! Gets the number of elements this simulation needs to calculate
    virtual size_t numberOfElements() const = 0;

    ProgressHandler& progress();

private:
    void initialize();

    void runSingleSimulation(const ProcessedSample& re_sample, size_t batch_start,
                             size_t batch_size, double weight = 1.0);

    //! Generate a single threaded computation for a given range of simulation elements
    //! @param re_sample Preprocessed version of our sample
    //! @param start Index of the first element to include into computation
    //! @param n_elements Number of elements to process
    virtual std::unique_ptr<IComputation> createComputation(const ProcessedSample& re_sample,
                                                            size_t start, size_t n_elements) = 0;

    //! Checks the distribution validity for simulation.
    virtual void validateParametrization(const ParameterDistribution&) const {}

    virtual void addBackgroundIntensity(size_t start_ind, size_t n_elements) = 0;

    //! Normalize the detector counts to beam intensity, to solid angle, and to exposure angle.
    //! @param start_ind Index of the first element to operate on
    //! @param n_elements Number of elements to process
    virtual void normalize(size_t start_ind, size_t n_elements) = 0;

    virtual void addDataToCache(double weight) = 0;

    virtual void moveDataFromCache() = 0;

    // used in MPI calculations for transfer of partial results
    virtual std::vector<double> rawResults() const = 0;
    virtual void setRawResults(const std::vector<double>& raw_data) = 0;

    std::shared_ptr<SimulationOptions> m_options; // TODO check whether still used
    std::shared_ptr<ProgressHandler> m_progress;
    std::shared_ptr<MultiLayer> m_sample;
    DistributionHandler m_distribution_handler;
    Instrument m_instrument;
    std::shared_ptr<IBackground> m_background;
};

#endif // BORNAGAIN_CORE_SIMULATION_ISIMULATION_H
#endif // USER_API
