//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Device/Coord/CoordSystem2D.cpp
//! @brief     Implements ICoordSystem classes.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Device/Coord/CoordSystem2D.h"
#include "Base/Axis/FixedBinAxis.h"
#include "Base/Const/Units.h"
#include "Base/Math/Constants.h"
#include "Base/Pixel/RectangularPixel.h"
#include "Base/Utils/Assert.h"
#include "Device/Coord/AxisNames.h"
#include <algorithm>
#include <cmath>

namespace {

double getQ(double wavelength, double angle)
{
    return 4.0 * M_PI * std::sin(angle) / wavelength;
}

double axisAngle(size_t i_axis, R3 k_f)
{
    if (i_axis == 0)
        return k_f.phi();
    if (i_axis == 1)
        return M_PI_2 - k_f.theta();
    ASSERT(0);
}

} // namespace

//  ************************************************************************************************
//  class CoordSystem2D
//  ************************************************************************************************

CoordSystem2D::CoordSystem2D(const Direction& direction, double wavelength)
    : m_wavelength(wavelength), m_alpha_i(-direction.alpha()), m_phi_i(direction.phi())
{
}

CoordSystem2D::CoordSystem2D(const CoordSystem2D& other)
    : m_axes(other.m_axes)
    , m_wavelength(other.m_wavelength)
    , m_alpha_i(other.m_alpha_i)
    , m_phi_i(other.m_phi_i)
{
}

void CoordSystem2D::addAxisData(std::string name, double min, double max,
                                Axes::Coords default_units, size_t nbins)
{
    m_axes.emplace_back(AxisData{name, min, max, default_units, nbins});
}

double CoordSystem2D::calculateMin(size_t i_axis, Axes::Coords units) const
{
    ASSERT(i_axis < dimension());
    units = substituteDefaultUnits(units);
    const auto& axis_data = m_axes[i_axis];
    if (units == Axes::Coords::NBINS)
        return 0.0;
    return calculateValue(i_axis, units, axis_data.min);
}

double CoordSystem2D::calculateMax(size_t i_axis, Axes::Coords units) const
{
    ASSERT(i_axis < dimension());
    units = substituteDefaultUnits(units);
    const auto& axis_data = m_axes[i_axis];
    if (units == Axes::Coords::NBINS)
        return static_cast<double>(axis_data.nbins);
    return calculateValue(i_axis, units, axis_data.max);
}

size_t CoordSystem2D::axisSize(size_t i_axis) const
{
    ASSERT(i_axis < dimension());
    return m_axes[i_axis].nbins;
}

std::vector<Axes::Coords> CoordSystem2D::availableUnits() const
{
    return {Axes::Coords::NBINS, Axes::Coords::RADIANS, Axes::Coords::DEGREES};
}

std::unique_ptr<const IAxis> CoordSystem2D::createConvertedAxis(size_t i_axis,
                                                                Axes::Coords units) const
{
    const double min = calculateMin(i_axis, units);
    const double max = calculateMax(i_axis, units);
    const auto& axis_name = axisName(i_axis, units);
    const auto axis_size = axisSize(i_axis);
    return std::make_unique<FixedBinAxis>(axis_name, axis_size, min, max);
}

void CoordSystem2D::addAxis(const IAxis& axis)
{
    const size_t iNewAxis = m_axes.size();
    addAxisData(axisName(iNewAxis), axis.lowerBound(), axis.upperBound(), defaultUnits(),
                axis.size());
}

//  ************************************************************************************************
//  class SphericalCoords
//  ************************************************************************************************

SphericalCoords::SphericalCoords(const CloneableVector<IAxis>& axes, const Direction& direction,
                                 double wavelength)
    : CoordSystem2D(direction, wavelength)
{
    ASSERT(axes.size() == 2);

    addAxis(*axes[0]);
    addAxis(*axes[1]);
}

SphericalCoords::SphericalCoords(const SphericalCoords& other) : CoordSystem2D(other) {}

SphericalCoords::~SphericalCoords() = default;

SphericalCoords* SphericalCoords::clone() const
{
    return new SphericalCoords(*this);
}

std::vector<Axes::Coords> SphericalCoords::availableUnits() const
{
    auto result = CoordSystem2D::availableUnits();
    result.push_back(Axes::Coords::QSPACE);
    return result;
}

double SphericalCoords::calculateValue(size_t i_axis, Axes::Coords units, double value) const
{
    switch (units) {
    case Axes::Coords::RADIANS:
        return value;
    case Axes::Coords::DEGREES:
        return Units::rad2deg(value);
    case Axes::Coords::QSPACE: {
        const R3 k_i = vecOfLambdaAlphaPhi(m_wavelength, m_alpha_i, m_phi_i);
        if (i_axis == 0) {
            const R3 k_f = vecOfLambdaAlphaPhi(m_wavelength, 0.0, value);
            return (k_i - k_f).y();
        }
        if (i_axis == 1) {
            const R3 k_f = vecOfLambdaAlphaPhi(m_wavelength, value, 0.0);
            return (k_f - k_i).z();
        }
        ASSERT(0);
    }
    case Axes::Coords::QXQY: {
        const R3 k_i = vecOfLambdaAlphaPhi(m_wavelength, m_alpha_i, m_phi_i);
        if (i_axis == 0) {
            const R3 k_f = vecOfLambdaAlphaPhi(m_wavelength, 0.0, value);
            return (k_i - k_f).y();
        }
        if (i_axis == 1) {
            const R3 k_f = vecOfLambdaAlphaPhi(m_wavelength, value, 0.0);
            return (k_f - k_i).x();
        }
        ASSERT(0);
    }
    default:
        ASSERT(0);
    }
}

std::vector<std::map<Axes::Coords, std::string>> SphericalCoords::createNameMaps() const
{
    return {DataUtils::AxisNames::sphericalAxis0, DataUtils::AxisNames::sphericalAxis1};
}

//  ************************************************************************************************
//  class ImageCoords
//  ************************************************************************************************

ImageCoords::ImageCoords(const CloneableVector<IAxis>& axes,
                         const RectangularPixel& regionOfInterestPixel, const Direction& direction,
                         double wavelength)
    : CoordSystem2D(direction, wavelength)
{
    ASSERT(axes.size() == 2);

    addAxis(*axes[0]);
    addAxis(*axes[1]);
    m_detector_pixel.reset(regionOfInterestPixel.clone());
}

ImageCoords::ImageCoords(const ImageCoords& other)
    : CoordSystem2D(other), m_detector_pixel(other.m_detector_pixel->clone())
{
}

ImageCoords::~ImageCoords() = default;

ImageCoords* ImageCoords::clone() const
{
    return new ImageCoords(*this);
}

std::vector<Axes::Coords> ImageCoords::availableUnits() const
{
    auto result = CoordSystem2D::availableUnits();
    result.push_back(Axes::Coords::QSPACE);
    result.push_back(Axes::Coords::MM);
    return result;
}

double ImageCoords::calculateValue(size_t i_axis, Axes::Coords units, double value) const
{
    ASSERT(m_wavelength > 0);
    if (units == Axes::Coords::MM)
        return value;
    const auto k00 = m_detector_pixel->getPosition(0.0, 0.0);
    const auto k01 = m_detector_pixel->getPosition(0.0, 1.0);
    const auto k10 = m_detector_pixel->getPosition(1.0, 0.0);
    const auto& max_pos = i_axis == 0 ? k10 : k01; // position of max along given axis
    const double shift = value - m_axes[i_axis].min;
    const R3 out_dir = k00 + shift * (max_pos - k00).unit();
    const R3 k_f = out_dir.unit() * M_TWOPI / m_wavelength;

    switch (units) {
    case Axes::Coords::RADIANS:
        return axisAngle(i_axis, k_f);
    case Axes::Coords::DEGREES:
        return Units::rad2deg(axisAngle(i_axis, k_f));
    case Axes::Coords::QSPACE: {
        const auto k_i = vecOfLambdaAlphaPhi(m_wavelength, m_alpha_i, m_phi_i);
        if (i_axis == 0)
            return (k_i - k_f).y();
        if (i_axis == 1)
            return (k_f - k_i).z();
    } break;
    case Axes::Coords::QXQY: {
        const auto k_i = vecOfLambdaAlphaPhi(m_wavelength, m_alpha_i, m_phi_i);
        if (i_axis == 0)
            return (k_i - k_f).y();
        if (i_axis == 1)
            return (k_f - k_i).x();
    } break;
    default:
        break;
    }
    ASSERT(0);
}

std::vector<std::map<Axes::Coords, std::string>> ImageCoords::createNameMaps() const
{
    return {DataUtils::AxisNames::rectangularAxis0, DataUtils::AxisNames::rectangularAxis1};
}

//  ************************************************************************************************
//  class OffSpecularCoordinates
//  ************************************************************************************************

OffSpecularCoordinates::OffSpecularCoordinates(const Direction& direction, const IAxis& alpha_axis,
                                               double alpha_f_min, double alpha_f_max, size_t nbins)
    : CoordSystem2D(direction)
{
    addAxisData(axisName(0), alpha_axis.lowerBound(), alpha_axis.upperBound(), defaultUnits(),
                alpha_axis.size());

    addAxisData(axisName(1), alpha_f_min, alpha_f_max, defaultUnits(), nbins);
}

OffSpecularCoordinates::OffSpecularCoordinates(const OffSpecularCoordinates& other)
    : CoordSystem2D(other)
{
}

OffSpecularCoordinates* OffSpecularCoordinates::createForRectangularDetector(
    const Direction& direction, const IAxis& alphaAxis, const RectangularPixel& detectorPixel,
    const IAxis& yAxis)
{
    const auto k00 = detectorPixel.getPosition(0.0, 0.0);
    const auto k01 = detectorPixel.getPosition(0.0, 1.0);
    const double alpha_f_min = M_PI_2 - k00.theta();
    const double alpha_f_max = M_PI_2 - k01.theta();

    return new OffSpecularCoordinates(direction, alphaAxis, alpha_f_min, alpha_f_max, yAxis.size());
}

OffSpecularCoordinates*
OffSpecularCoordinates::createForSphericalDetector(const Direction& direction,
                                                   const IAxis& alphaAxis, const IAxis& yAxis)
{
    const double alpha_f_min = yAxis.lowerBound();
    const double alpha_f_max = yAxis.upperBound();

    return new OffSpecularCoordinates(direction, alphaAxis, alpha_f_min, alpha_f_max, yAxis.size());
}

OffSpecularCoordinates* OffSpecularCoordinates::clone() const
{
    return new OffSpecularCoordinates(*this);
}

Axes::Coords OffSpecularCoordinates::defaultUnits() const
{
    return Axes::Coords::DEGREES;
}

double OffSpecularCoordinates::calculateValue(size_t, Axes::Coords units, double value) const
{
    switch (units) {
    case Axes::Coords::RADIANS:
        return value;
    case Axes::Coords::DEGREES:
        return Units::rad2deg(value);
    default:
        ASSERT(0);
    }
}

std::vector<std::map<Axes::Coords, std::string>> OffSpecularCoordinates::createNameMaps() const
{
    return {DataUtils::AxisNames::offSpecularAxis0, DataUtils::AxisNames::offSpecularAxis1};
}

//  ************************************************************************************************
//  class DepthProbeCoordinates
//  ************************************************************************************************

const std::string z_axis_name = "Position [nm]";

DepthProbeCoordinates::DepthProbeCoordinates(const Direction& direction, double wavelength,
                                             const IAxis& alpha_axis, const IAxis& z_axis)
    : CoordSystem2D(direction, wavelength)
{
    const auto& alpha_axis_name = axisName(0);
    const auto& z_axis_name = axisName(1);
    addAxisData(alpha_axis_name, alpha_axis.lowerBound(), alpha_axis.upperBound(), defaultUnits(),
                alpha_axis.size());
    addAxisData(z_axis_name, z_axis.lowerBound(), z_axis.upperBound(), defaultUnits(),
                z_axis.size());
}

DepthProbeCoordinates::DepthProbeCoordinates(const DepthProbeCoordinates& other)
    : CoordSystem2D(other)
{
}

DepthProbeCoordinates::~DepthProbeCoordinates() = default;

DepthProbeCoordinates* DepthProbeCoordinates::clone() const
{
    return new DepthProbeCoordinates(*this);
}

std::vector<Axes::Coords> DepthProbeCoordinates::availableUnits() const
{
    auto result = CoordSystem2D::availableUnits();
    result.push_back(Axes::Coords::QSPACE);
    return result;
}

double DepthProbeCoordinates::calculateValue(size_t i_axis, Axes::Coords units, double value) const
{
    const auto& available_units = availableUnits();
    if (std::find(available_units.begin(), available_units.end(), units) == available_units.cend())
        ASSERT(0);

    if (i_axis == 1)
        return value; // unit conversions are not applied to sample position axis
    switch (units) {
    case Axes::Coords::DEGREES:
        return Units::rad2deg(value);
    case Axes::Coords::QSPACE:
        return getQ(m_wavelength, value);
    default:
        return value;
    }
}

std::vector<std::map<Axes::Coords, std::string>> DepthProbeCoordinates::createNameMaps() const
{
    return {DataUtils::AxisNames::specAxis, DataUtils::AxisNames::sampleDepthAxis};
}
