//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Application/Application.cpp
//! @brief     Implements class Application
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Application/Application.h"
#include <QApplication>
#include <QFile>
#include <QStandardPaths>

GlobalSettings* baApp; //!< global pointer to _the_ instance

GlobalSettings::GlobalSettings()
    : m_currentStyle(ApplicationSettings::Style::native)
{
    baApp = this;
}

ApplicationSettings& GlobalSettings::settings()
{
    return m_settings;
}

void GlobalSettings::loadStyle(ApplicationSettings::Style style)
{
    QString stylesheet;

    switch (style) {
    case ApplicationSettings::Style::light: {
        QFile base(":/styles/Base.stylesheet");
        base.open(QFile::ReadOnly);
        QFile light(":/styles/Light.stylesheet");
        light.open(QFile::ReadOnly);
        stylesheet = base.readAll() + light.readAll();
        break;
    }
    case ApplicationSettings::Style::dark: {
        QFile base(":/styles/Base.stylesheet");
        base.open(QFile::ReadOnly);
        QFile dark(":/styles/Dark.stylesheet");
        dark.open(QFile::ReadOnly);
        stylesheet = base.readAll() + dark.readAll();
        break;
    }
    default: {
        QFile native(":/styles/Native.stylesheet");
        native.open(QFile::ReadOnly);
        stylesheet = native.readAll();
        break;
    }
    }

    qApp->setStyleSheet(stylesheet);
    m_currentStyle = style;

    // -- init palette for later usage; could be improved by parsing the stylesheet
    switch (m_currentStyle) {
    case ApplicationSettings::Style::light: {
        m_styleSheetPalette.headlineText = QColor(48, 113, 145);
        m_styleSheetPalette.headlineBackground = QColor(222, 232, 237);
        break;
    }
    case ApplicationSettings::Style::dark: {
        m_styleSheetPalette.headlineText = QColor(213, 220, 223);
        m_styleSheetPalette.headlineBackground = QColor(29, 39, 44);
        break;
    }
    default: {
        m_styleSheetPalette.headlineText = Qt::black;
        m_styleSheetPalette.headlineBackground = QColor(250, 250, 250);
        break;
    }
    }
}

ApplicationSettings::Style GlobalSettings::currentStyle()
{
    return m_currentStyle;
}

const GlobalSettings::Palette& GlobalSettings::styleSheetPalette() const
{
    return m_styleSheetPalette;
}

InstrumentLibrary& GlobalSettings::instrumentLibrary()
{
    return m_instrumentLibrary;
}

QString GlobalSettings::instrumentLibraryFilePath() const
{
    return appDataFolder() + "/BornAgainInstrumentLibrary.balib";
}

QString GlobalSettings::appDataFolder() const
{
    return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation);
}
