//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Application/ApplicationSettings.cpp
//! @brief     Implements class ApplicationSettings
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Application/ApplicationSettings.h"
#include "Base/Utils/Assert.h"
#include <QSettings>
#include <QWidget>

namespace {
const QString S_CREATE_NEW_PROJECT_ON_STARTUP = "CreateNewProjectOnStartup";
const QString S_STYLE = "Style";
const QString S_SIZE = "Size";
const QString S_POS = "Pos";
const QString S_DEFAULT_FUNCTIONALITIES = "DefaultFunctionalities";
const QString S_SINGLE_INSTRUMENT_MODE = "SingleInstrumentMode";
const QString S_SINGLE_SAMPLE_MODE = "SingleSampleMode";
const QString S_DEFAULT_UNIT_IS_ANGSTROM = "DefaultUnitIsAngstrom";
} // namespace

bool ApplicationSettings::useNativeFileDialog() const
{
#ifdef _WIN32
    return true;
#else
    return false;
#endif
}

bool ApplicationSettings::createNewProjectOnStartup() const
{
    return QSettings().value(S_CREATE_NEW_PROJECT_ON_STARTUP, true).toBool();
}

void ApplicationSettings::setCreateNewProjectOnStartup(bool b)
{
    QSettings().setValue(S_CREATE_NEW_PROJECT_ON_STARTUP, b);
}

ApplicationSettings::Style ApplicationSettings::styleToUse() const
{
    return static_cast<Style>(QSettings().value(S_STYLE, static_cast<int>(Style::native)).toInt());
}

void ApplicationSettings::setStyleToUse(Style style)
{
    QSettings().setValue(S_STYLE, static_cast<int>(style));
}

QVariant ApplicationSettings::defaultFunctionalities(const QVariant& absenceValue) const
{
    return QSettings().value(S_DEFAULT_FUNCTIONALITIES, absenceValue);
}

void ApplicationSettings::setDefaultFunctionalities(const QVariant& functionalities)
{
    QSettings().setValue(S_DEFAULT_FUNCTIONALITIES, functionalities);
}

bool ApplicationSettings::defaultIsSingleInstrumentMode() const
{
    return QSettings().value(S_SINGLE_INSTRUMENT_MODE, false).toBool();
}

void ApplicationSettings::setDefaultIsSingleInstrumentMode(bool b)
{
    QSettings().setValue(S_SINGLE_INSTRUMENT_MODE, b);
}

bool ApplicationSettings::defaultIsSingleSampleMode() const
{
    return QSettings().value(S_SINGLE_SAMPLE_MODE, false).toBool();
}

void ApplicationSettings::setDefaultIsSingleSampleMode(bool b)
{
    QSettings().setValue(S_SINGLE_SAMPLE_MODE, b);
}

bool ApplicationSettings::defaultUnitIsAngstrom() const
{
    return QSettings().value(S_DEFAULT_UNIT_IS_ANGSTROM, false).toBool();
}

void ApplicationSettings::setDefaultUnitIsAngstrom(bool b) const
{
    QSettings().setValue(S_DEFAULT_UNIT_IS_ANGSTROM, b);
}

void ApplicationSettings::saveWindowSizeAndPos(const QWidget* w)
{
    ASSERT(!w->objectName().isEmpty());
    QSettings settings;
    settings.setValue(S_SIZE + "/" + w->objectName(), w->size());
    settings.setValue(S_POS + "/" + w->objectName(), w->pos());
}

void ApplicationSettings::loadWindowSizeAndPos(QWidget* w)
{
    ASSERT(!w->objectName().isEmpty());

    QSettings settings;
    const QSize size = settings.value(S_SIZE + "/" + w->objectName(), QSize()).toSize();
    if (size.isValid())
        w->resize(size);
    if (settings.contains(S_POS + "/" + w->objectName()))
        w->move(settings.value(S_POS + "/" + w->objectName()).toPoint());
}
