//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/CommonWidgets/DoubleSpinBox.h
//! @brief     Defines class DoubleSpinBox
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_COMMONWIDGETS_DOUBLESPINBOX_H
#define BORNAGAIN_GUI_COMMONWIDGETS_DOUBLESPINBOX_H

#include "GUI/Models/Types/DoubleDescriptor.h"
#include <QDoubleSpinBox>

//! SpinBox for DoubleDescriptors, supporting units.
class DoubleSpinBox : public QDoubleSpinBox {
    Q_OBJECT
public:
    DoubleSpinBox(QWidget* parent, const DoubleDescriptor& d);

    //! Set a display unit.
    //!
    //! The caller has to make sure that the new display unit has a conversion to/from the contained
    //! base value unit.
    void setDisplayUnit(Unit displayUnit);

    //! Set the base value (unit is the one of the contained descriptor).
    void setBaseValue(double baseValue);

    //! The display unit as human readable string.
    QString displayUnitAsString() const;

    //! The descriptor on which this spinbox operates.
    const DoubleDescriptor& valueDescriptor() const;

    //! Returns the unit of the contained DoubleDescriptor.
    //!
    //! If the unit is defined as a string, this method returns Unit::other. To get the string, use
    //! valueDescriptor().unit
    Unit baseUnit() const;

signals:
    //! Emitted whenever the value changes.
    //!
    //! newBaseValue is in the unit of the valueDescriptor.
    void baseValueChanged(double newBaseValue);

protected:
    virtual void wheelEvent(QWheelEvent* event) override;

private:
    void onDisplayValueChanged(double newValue);

    double toDisplayValue(double baseValue) const;
    double toBaseValue(double displayValue) const;

    using QDoubleSpinBox::setValue; // To hide from usage

private:
    Unit m_displayUnit = Unit::unitless;

    DoubleDescriptor m_valueDescriptor;

    //! it was decided to not show the unit as a suffix. However, this may be user
    //! selectable once, therefore the code is kept and controlled by this flag
    bool m_showUnitAsSuffix = false;
};


#endif // BORNAGAIN_GUI_COMMONWIDGETS_DOUBLESPINBOX_H
