//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Items/DetectorItems.cpp
//! @brief     Implements classes DetectorItems
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Items/DetectorItems.h"
#include "Device/Detector/IDetector2D.h"
#include "Device/Mask/IShape2D.h"
#include "Device/Resolution/ResolutionFunction2DGaussian.h"
#include "GUI/Items/MaskItems.h"
#include "GUI/Items/ResolutionFunctionItems.h"
#include "GUI/Items/SessionModel.h"
#include "GUI/Items/SphericalDetectorItem.h"
#include "GUI/Models/Group/GroupInfo.h"

namespace {
const QString res_func_group_label = "Type";

const GroupInfo createResolutionFunctionGroup()
{
    GroupInfo info;
    info.add(ResolutionFunctionNoneItem::M_TYPE, "None");
    info.add(ResolutionFunction2DGaussianItem::M_TYPE, "2D Gaussian");
    info.setDefaultType(ResolutionFunctionNoneItem::M_TYPE);
    return info;
}

} // namespace

std::unique_ptr<IDetector2D> DetectorItem::createDetector() const
{
    auto result = createDomainDetector();
    addMasksToDomain(result.get());

    if (auto resFunc = createResolutionFunction())
        result->setResolutionFunction(*resFunc);

    return result;
}

void DetectorItem::clearMasks()
{
    if (auto maskContainer = maskContainerItem())
        delete takeRow(rowOfChild(maskContainer));
}

MaskContainerItem* DetectorItem::maskContainerItem() const
{
    return dynamic_cast<MaskContainerItem*>(getItem(T_MASKS));
}

void DetectorItem::createMaskContainer()
{
    if (!maskContainerItem())
        model()->insertItem<MaskContainerItem>(this);
}

void DetectorItem::importMasks(const MaskContainerItem* maskContainer)
{
    clearMasks();

    if (maskContainer)
        model()->copyItem(maskContainer, this, T_MASKS);
}

SessionItem* DetectorItem::resolutionFunctionItem() const
{
    return getItem(P_RESOLUTION_FUNCTION);
}

ResolutionFunctionItem* DetectorItem::resolutionFunction() const
{
    return dynamic_cast<ResolutionFunctionItem*>(getGroupItem(P_RESOLUTION_FUNCTION));
}

DetectorItem::DetectorItem(const QString& modelType) : SessionItem(modelType)
{
    registerTag(T_MASKS, 0, -1, {MaskContainerItem::M_TYPE});
    setDefaultTag(T_MASKS);

    mapper()->setOnPropertyChange([this](const QString& name) {
        if (name == P_RESOLUTION_FUNCTION)
            update_resolution_function_tooltips();
    });
}

void DetectorItem::register_resolution_function()
{
    static const GroupInfo resolutionFunctionGroup = createResolutionFunctionGroup();
    auto item = addGroupProperty(P_RESOLUTION_FUNCTION, resolutionFunctionGroup);
    item->setDisplayName(res_func_group_label);
    item->setToolTip("Detector resolution function");
}

void DetectorItem::update_resolution_function_tooltips()
{
    auto& resfuncItem = groupItem<ResolutionFunctionItem>(DetectorItem::P_RESOLUTION_FUNCTION);

    if (resfuncItem.modelType() == ResolutionFunction2DGaussianItem::M_TYPE) {
        QString units = modelType() == SphericalDetectorItem::M_TYPE ? "deg" : "mm";

        ResolutionFunction2DGaussianItem& resfuncGaussItem =
            dynamic_cast<ResolutionFunction2DGaussianItem&>(resfuncItem);
        resfuncGaussItem.sigmaXItem()->setToolTip("Resolution along horizontal axis (in " + units
                                                  + ")");
        resfuncGaussItem.sigmaYItem()->setToolTip("Resolution along vertical axis (in " + units
                                                  + ")");
    }
}

std::unique_ptr<IResolutionFunction2D> DetectorItem::createResolutionFunction() const
{
    auto& resfuncItem = groupItem<ResolutionFunctionItem>(DetectorItem::P_RESOLUTION_FUNCTION);
    return resfuncItem.createResolutionFunction(axesToDomainUnitsFactor());
}

void DetectorItem::addMasksToDomain(IDetector2D* detector) const
{
    auto maskContainer = maskContainerItem();

    if (!maskContainer)
        return;

    const double scale = axesToDomainUnitsFactor();

    for (int i_row = maskContainer->children().size(); i_row > 0; --i_row) {
        if (auto maskItem = dynamic_cast<MaskItem*>(maskContainer->children().at(i_row - 1))) {

            if (maskItem->modelType() == RegionOfInterestItem::M_TYPE) {
                RectangleItem* rectItem = static_cast<RectangleItem*>(maskItem);
                double xlow = scale * rectItem->xLow();
                double ylow = scale * rectItem->yLow();
                double xup = scale * rectItem->xUp();
                double yup = scale * rectItem->yUp();
                detector->setRegionOfInterest(xlow, ylow, xup, yup);
            } else {
                std::unique_ptr<IShape2D> shape(maskItem->createShape(scale));
                bool mask_value = maskItem->maskValue();
                detector->addMask(*shape, mask_value);
            }
        }
    }
}
