//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Items/MultiLayerItem.cpp
//! @brief     Implements class MultiLayerItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Items/MultiLayerItem.h"
#include "GUI/Items/LayerItem.h"
#include "GUI/Items/SessionModel.h"
#include "GUI/Models/Types/VectorDescriptor.h"

namespace {
const QString external_field_tooltip = "External field (A/m)";
}

MultiLayerItem::MultiLayerItem() : SessionGraphicsItem(M_TYPE)
{
    setToolTip("A multilayer to hold stack of layers");
    setItemName(M_TYPE);

    addProperty(P_CROSS_CORR_LENGTH, 0.0)
        ->setDecimals(5)
        .setToolTip("Cross correlation length of roughnesses \n"
                    "between interfaces in nanometers");
    addProperty<VectorItem>(P_EXTERNAL_FIELD)->setToolTip(external_field_tooltip);
    addProperty(P_DESCRIPTION, QString())->setVisible(false);

    registerTag(T_LAYERS, 0, -1, {LayerItem::M_TYPE});
    setDefaultTag(T_LAYERS);

    mapper()->setOnChildrenChange([this](SessionItem*) { updateLayers(); });
}

QVector<ItemWithMaterial*> MultiLayerItem::itemsWithMaterial()
{
    QVector<ItemWithMaterial*> result;
    for (auto layer_item : getItems(T_LAYERS))
        result.append(dynamic_cast<LayerItem*>(layer_item)->itemsWithMaterial());
    return result;
}

QString MultiLayerItem::description() const
{
    return getItemValue(P_DESCRIPTION).toString();
}

void MultiLayerItem::setDescription(const QString& description)
{
    setItemValue(P_DESCRIPTION, description);
}

DoubleDescriptor MultiLayerItem::crossCorrLength() const
{
    DoubleDescriptor d(getItem(P_CROSS_CORR_LENGTH), Unit::nanometer);
    d.tooltip = "Cross correlation length of roughnesses between interfaces";
    return d;
}

bool MultiLayerItem::isCrossCorrLengthPropertyName(const QString& name)
{
    return name == P_CROSS_CORR_LENGTH;
}

R3 MultiLayerItem::externalField() const
{
    return item<VectorItem>(P_EXTERNAL_FIELD)->getVector();
}

void MultiLayerItem::setExternalField(const R3& ext_field)
{
    item<VectorItem>(P_EXTERNAL_FIELD)->setVector(ext_field);
}

SessionItem* MultiLayerItem::externalFieldItem() const
{
    return getItem(P_EXTERNAL_FIELD);
}

VectorDescriptor MultiLayerItem::externalFieldVector() const
{
    return VectorDescriptor(item<VectorItem>(P_EXTERNAL_FIELD), "A/m");
}

QVector<LayerItem*> MultiLayerItem::layers() const
{
    return items<LayerItem>(T_LAYERS);
}

SessionItem* MultiLayerItem::layersItem() const
{
    return getItem(T_LAYERS);
}

LayerItem* MultiLayerItem::addLayer(int index)
{
    return model()->insertItem<LayerItem>(this, index);
}

void MultiLayerItem::removeLayer(LayerItem* item)
{
    model()->removeItem(item);
}


void MultiLayerItem::moveLayer(LayerItem* item, LayerItem* beforeThisLayer)
{
    int index = -1; // move to end
    if (beforeThisLayer != nullptr) {
        index = layers().indexOf(beforeThisLayer);
    }

    model()->moveItem(item, this, index);
}

void MultiLayerItem::updateLayers()
{
    QVector<LayerItem*> list = childrenOfType<LayerItem>();
    for (auto it = list.begin(); it != list.end(); ++it) {
        (*it)->setRoughnessEnabled(it != list.begin());

        if (it == list.begin() || it == (list.end() - 1)) {
            (*it)->setThicknessEnabled(false);
            (*it)->thickness().set(0.0);
        } else
            (*it)->setThicknessEnabled(true);
    }
}
