//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Items/ParticleLayoutItem.cpp
//! @brief     Implements class ParticleLayoutItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Items/ParticleLayoutItem.h"
#include "GUI/Items/InterferenceItems.h"
#include "GUI/Items/ItemCatalog.h"
#include "GUI/Items/Lattice2DItems.h"
#include "GUI/Items/MesoCrystalItem.h"
#include "GUI/Items/ParticleCompositionItem.h"
#include "GUI/Items/ParticleCoreShellItem.h"
#include "GUI/Items/ParticleItem.h"
#include "GUI/Items/SessionModel.h"
#include "GUI/Models/Types/DoubleDescriptor.h"

namespace {

//! Returns true if name is related to 2D interference functions.
bool isInterference2D(const QString& model_type)
{
    if (model_type == Interference2DLatticeItem::M_TYPE
        || model_type == Interference2DParaCrystalItem::M_TYPE
        || model_type == InterferenceFinite2DLatticeItem::M_TYPE
        || model_type == InterferenceHardDiskItem::M_TYPE)
        return true;
    return false;
}

//! Returns true if name is related to 2D interference functions.
bool isLattice2D(SessionItem* item)
{
    return dynamic_cast<Lattice2DItem*>(item);
}

const QString density_tooltip =
    "Number of particles per square nanometer (particle surface density).\n "
    "Should be defined for disordered and 1d-ordered particle collections.";
const QString weight_tooltip =
    "Weight of this particle layout.\n"
    "Should be used when multiple layouts define different domains in the sample.";
} // namespace

ParticleLayoutItem::ParticleLayoutItem() : SessionGraphicsItem(M_TYPE)
{
    setToolTip("A layout of particles");

    addProperty(P_TOTAL_DENSITY, 0.01)->setToolTip(density_tooltip);
    getItem(P_TOTAL_DENSITY)->setDecimals(10);
    addProperty(P_WEIGHT, 1.0)->setToolTip(weight_tooltip);

    registerTag(T_PARTICLES, 0, -1,
                {ParticleItem::M_TYPE, ParticleCoreShellItem::M_TYPE,
                 ParticleCompositionItem::M_TYPE, MesoCrystalItem::M_TYPE});
    setDefaultTag(T_PARTICLES);
    registerTag(T_INTERFERENCE, 0, 1,
                {Interference1DLatticeItem::M_TYPE, Interference2DLatticeItem::M_TYPE,
                 Interference2DParaCrystalItem::M_TYPE, InterferenceFinite2DLatticeItem::M_TYPE,
                 InterferenceHardDiskItem::M_TYPE, InterferenceRadialParaCrystalItem::M_TYPE});

    mapper()->setOnChildrenChange([this](SessionItem*) {
        updateDensityAppearance();
        updateDensityValue();
    });

    mapper()->setOnAnyChildChange([this](SessionItem* item) {
        if (isLattice2D(item) || (item && isLattice2D(item->parent())))
            updateDensityValue();
    });
}

DoubleDescriptor ParticleLayoutItem::totalDensity() const
{
    DoubleDescriptor d(getItem(P_TOTAL_DENSITY), Unit::nanometerPowerMinus2);
    d.tooltip = "Number of particles per area (particle surface density).\n "
                "Should be defined for disordered and 1d-ordered particle collections.";
    return d;
}

DoubleDescriptor ParticleLayoutItem::weight() const
{
    return DoubleDescriptor(getItem(P_WEIGHT), Unit::unitless);
}

QVector<ItemWithParticles*> ParticleLayoutItem::particles() const
{
    return items<ItemWithParticles>(T_PARTICLES);
}

void ParticleLayoutItem::addParticle(ItemWithParticles* particle)
{
    model()->moveItem(particle, this, -1, T_PARTICLES);
}

void ParticleLayoutItem::removeParticle(ItemWithParticles* particle)
{
    model()->removeItem(particle);
}

SelectionDescriptor<InterferenceItem*> ParticleLayoutItem::interference() const
{
    SelectionDescriptor<InterferenceItem*> d;

    // we need a special filling for this selection descriptor (not just from a GroupItem), since
    // the interference is stored in a child, which can be present or not.

    static QVector<QPair<QString, QString>> map;

    // initialize if not done so far
    if (map.isEmpty()) {
        map << qMakePair(QString("None"), QString(""));
        for (auto modelType : ItemCatalog::interferenceFunctionTypes())
            map << qMakePair(ItemCatalog::instance().uiInfo(modelType).menuEntry, modelType);
    }

    d.label = "Interference function";

    for (auto [title, type] : map)
        d.options << title;

    d.currentItem = [=] { return dynamic_cast<InterferenceItem*>(getItem(T_INTERFERENCE)); };

    d.currentIndexSetter = [=](int current) {
        if (auto item = getItem(T_INTERFERENCE))
            model()->removeItem(item);
        if (current > 0)
            model()->insertNewItem(map[current].second, const_cast<ParticleLayoutItem*>(this), -1,
                                   T_INTERFERENCE);
    };

    d.currentIndexGetter = [=]() {
        if (auto item = dynamic_cast<InterferenceItem*>(getItem(T_INTERFERENCE)))
            for (int i = 1; i < map.size(); i++)
                if (map[i].second == item->modelType())
                    return i;

        return 0;
    };

    return d;
}

void ParticleLayoutItem::setInterference(InterferenceItem* interference)
{
    model()->moveItem(interference, this, -1, T_INTERFERENCE);
}

//! Disables/enables total density property, depending on type of interference function.
//! Two dimensional interference calculates density automatically, so property should
//! be disabled.

void ParticleLayoutItem::updateDensityAppearance()
{
    getItem(P_TOTAL_DENSITY)->setEnabled(true);
    if (auto interferenceItem = getItem(T_INTERFERENCE))
        if (isInterference2D(interferenceItem->modelType()))
            getItem(P_TOTAL_DENSITY)->setEnabled(false);
}

//! Updates the value of TotalSurfaceDensity on lattice type change.

void ParticleLayoutItem::updateDensityValue()
{
    if (auto interferenceItem = getItem(T_INTERFERENCE)) {
        if (Interference2DAbstractLatticeItem* interLatticeItem =
                dynamic_cast<Interference2DAbstractLatticeItem*>(interferenceItem)) {
            Lattice2DItem* latticeItem = interLatticeItem->latticeType();
            double area = 0.0;
            try {
                area = latticeItem->unitCellArea();
            } catch (const std::exception&) {
                // nothing to do here; new exception will be caught during job execution
            }
            setItemValue(P_TOTAL_DENSITY, area == 0.0 ? 0.0 : 1.0 / area);
        } else if (InterferenceHardDiskItem* interHDItem =
                       dynamic_cast<InterferenceHardDiskItem*>(interferenceItem)) {
            double density = interHDItem->density();
            setItemValue(P_TOTAL_DENSITY, density);
        }
    }
}
