//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/Domain/TransformToDomain.cpp
//! @brief     Implements class TransformToDomain
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/Domain/TransformToDomain.h"
#include "Base/Vector/Transform3D.h"
#include "Core/Scan/AlphaScan.h"
#include "Core/Simulation/GISASSimulation.h"
#include "Device/Resolution/ScanResolution.h"
#include "GUI/Items/BeamAngleItems.h"
#include "GUI/Items/BeamItems.h"
#include "GUI/Items/BeamWavelengthItem.h"
#include "GUI/Items/JobItem.h"
#include "GUI/Items/JobModelFunctions.h"
#include "GUI/Items/LayerItem.h"
#include "GUI/Items/LayerRoughnessItems.h"
#include "GUI/Items/MaterialItemUtils.h"
#include "GUI/Items/MesoCrystalItem.h"
#include "GUI/Items/MultiLayerItem.h"
#include "GUI/Items/ParticleCompositionItem.h"
#include "GUI/Items/ParticleCoreShellItem.h"
#include "GUI/Items/ParticleItem.h"
#include "GUI/Items/ParticleLayoutItem.h"
#include "GUI/Items/RotationItems.h"
#include "GUI/Items/SimulationOptionsItem.h"
#include "GUI/Items/TransformationItem.h"
#include "GUI/Items/VectorItem.h"
#include "GUI/Models/Types/DoubleDescriptor.h"
#include "GUI/Models/Types/UIntDescriptor.h"
#include "Param/Distrib/RangedDistributions.h"
#include "Resample/Options/SimulationOptions.h"
#include "Sample/Particle/MesoCrystal.h"
#include "Sample/Particle/Particle.h"
#include "Sample/Particle/ParticleCoreShell.h"

namespace {
template <class T>
void setParameterDistributionToSimulation(ParameterDistribution::WhichParameter which,
                                          const SessionItem* item, ISimulation& simulation);

std::unique_ptr<ScanResolution> createScanResolution(const SessionItem* item);
} // namespace

std::unique_ptr<Material>
GUI::Transform::ToDomain::createDomainMaterial(const ItemWithMaterial& item)
{
    auto parent_job = GUI::Model::JobFunctions::findJobItem(&item);
    const MaterialItemContainer* container =
        parent_job ? parent_job->materialContainerItem() : nullptr;
    return container
               ? GUI::Model::MaterialItemUtils::createDomainMaterial(item.materialIdentifier(),
                                                                     *container)
               : GUI::Model::MaterialItemUtils::createDomainMaterial(item.materialIdentifier());
}

std::unique_ptr<MultiLayer> GUI::Transform::ToDomain::createMultiLayer(const MultiLayerItem& item)
{
    auto P_multilayer = std::make_unique<MultiLayer>();
    double cross_corr_length = item.crossCorrLength();
    if (cross_corr_length > 0)
        P_multilayer->setCrossCorrLength(cross_corr_length);
    R3 external_field = item.externalField();
    P_multilayer->setExternalField(external_field);
    return P_multilayer;
}

std::unique_ptr<Layer> GUI::Transform::ToDomain::createLayer(const LayerItem& item)
{
    auto P_layer = std::make_unique<Layer>(*createDomainMaterial(item), item.thickness());
    P_layer->setNumberOfSlices(item.numSlices());
    return P_layer;
}

std::unique_ptr<LayerRoughness> GUI::Transform::ToDomain::createLayerRoughness(
    const std::variant<LayerZeroRoughnessItem*, LayerBasicRoughnessItem*>& roughness)
{
    if (std::holds_alternative<LayerZeroRoughnessItem*>(roughness))
        return nullptr;
    if (std::holds_alternative<LayerBasicRoughnessItem*>(roughness)) {
        const auto& basicRoughnessItem = *std::get<LayerBasicRoughnessItem*>(roughness);
        return std::make_unique<LayerRoughness>(basicRoughnessItem.sigma(),
                                                basicRoughnessItem.hurst(),
                                                basicRoughnessItem.lateralCorrelationLength());
    }
    ASSERT(0);
}

std::unique_ptr<ParticleLayout>
GUI::Transform::ToDomain::createParticleLayout(const ParticleLayoutItem& item)
{
    auto P_layout = std::make_unique<ParticleLayout>();
    double total_density = item.totalDensity();
    double layout_weight = item.weight();
    P_layout->setTotalParticleSurfaceDensity(total_density);
    P_layout->setWeight(layout_weight);
    return P_layout;
}

std::unique_ptr<IParticle> GUI::Transform::ToDomain::createIParticle(const SessionItem& item)
{
    std::unique_ptr<IParticle> P_particle;
    if (item.hasModelType<ParticleItem>()) {
        auto& particle_item = dynamic_cast<const ParticleItem&>(item);
        P_particle = particle_item.createParticle();
    } else if (item.hasModelType<ParticleCoreShellItem>()) {
        auto& particle_coreshell_item = dynamic_cast<const ParticleCoreShellItem&>(item);
        P_particle = particle_coreshell_item.createParticleCoreShell();
    } else if (item.hasModelType<ParticleCompositionItem>()) {
        auto& particle_composition_item = dynamic_cast<const ParticleCompositionItem&>(item);
        P_particle = particle_composition_item.createParticleComposition();
    } else if (item.hasModelType<MesoCrystalItem>()) {
        auto& mesocrystal_item = dynamic_cast<const MesoCrystalItem&>(item);
        P_particle = mesocrystal_item.createMesoCrystal();
    }
    return P_particle;
}

//! adds DistributionParameters to the ISimulation
void GUI::Transform::ToDomain::addDistributionParametersToSimulation(const BeamItem& beam_item,
                                                                     GISASSimulation& simulation)
{
    if (beam_item.modelType() != GISASBeamItem::M_TYPE) {
        ASSERT(beam_item.modelType() == GISASBeamItem::M_TYPE);
        return;
    }

    setParameterDistributionToSimulation<BeamWavelengthItem>(
        ParameterDistribution::BeamWavelength, beam_item.wavelengthItem(), simulation);
    setParameterDistributionToSimulation<BeamInclinationAngleItem>(
        ParameterDistribution::BeamInclinationAngle, beam_item.inclinationAngleItem(), simulation);
    setParameterDistributionToSimulation<BeamAzimuthalAngleItem>(
        ParameterDistribution::BeamAzimuthalAngle, beam_item.azimuthalAngleItem(), simulation);
}

void GUI::Transform::ToDomain::addBeamDivergencesToScan(const BeamItem& beam_item, AlphaScan& scan)
{
    if (beam_item.modelType() != SpecularBeamItem::M_TYPE) {
        ASSERT(beam_item.modelType() == SpecularBeamItem::M_TYPE);
        return;
    }

    auto resolution = createScanResolution(beam_item.wavelengthItem());
    if (resolution)
        scan.setWavelengthResolution(*resolution);
    resolution = createScanResolution(beam_item.inclinationAngleItem());
    if (resolution)
        scan.setAngleResolution(*resolution);
}

void GUI::Transform::ToDomain::setBeamDistribution(ParameterDistribution::WhichParameter which,
                                                   const BeamDistributionItem& item,
                                                   ISimulation& simulation)
{
    auto P_par_distr = item.getParameterDistributionForName(which);
    if (P_par_distr)
        simulation.addParameterDistribution(*P_par_distr);
}

void GUI::Transform::ToDomain::setSimulationOptions(ISimulation* simulation,
                                                    const SessionItem& item)
{
    ASSERT(item.modelType() == SimulationOptionsItem::M_TYPE);

    if (auto optionItem = dynamic_cast<const SimulationOptionsItem*>(&item)) {
        simulation->options().setNumberOfThreads(optionItem->numberOfThreads());
        if (optionItem->useMonteCarloIntegration()) {
            simulation->options().setMonteCarloIntegration(true,
                                                           optionItem->numberOfMonteCarloPoints());
        }
        simulation->options().setUseAvgMaterials(optionItem->useAverageMaterials());
        simulation->options().setIncludeSpecular(optionItem->includeSpecularPeak());
    }
}

void GUI::Transform::ToDomain::setTransformationInfo(IParticle* result,
                                                     const ItemWithParticles& item)
{
    result->setPosition(item.position());
    const Transform3D r = item.rotation();
    if (!r.isIdentity()) {
        std::unique_ptr<IRotation> rotation(IRotation::createRotation(r));
        result->setRotation(*rotation);
    }
}

namespace {

//! If the given item has distributions, then add these distributions to the simulation's
//! distributions handler.
template <class T>
void setParameterDistributionToSimulation(ParameterDistribution::WhichParameter which,
                                          const SessionItem* item, ISimulation& simulation)
{
    const auto parameter_item = dynamic_cast<const T*>(item);
    if (!parameter_item) {
        ASSERT(parameter_item);
        return;
    }

    auto P_par_distr = parameter_item->getParameterDistributionForName(which);
    if (P_par_distr)
        simulation.addParameterDistribution(*P_par_distr);
}

std::unique_ptr<ScanResolution> createScanResolution(const SessionItem* item)
{
    auto beam_item = dynamic_cast<const BeamDistributionItem*>(item);
    if (!beam_item)
        return nullptr;

    auto distr_item = dynamic_cast<const SymmetricDistributionItem*>(beam_item->distribution());
    if (!distr_item)
        return nullptr;

    const double scale = beam_item->scaleFactor();
    auto ranged_distr = distr_item->createIRangedDistribution(scale);
    if (!ranged_distr)
        return nullptr;

    const double deviation = distr_item->deviation(scale);
    return std::unique_ptr<ScanResolution>(
        ScanResolution::scanAbsoluteResolution(*ranged_distr, deviation));
}
} // namespace
