//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/Material/MaterialModel.cpp
//! @brief     Implements class MaterialModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/Material/MaterialModel.h"
#include "GUI/Items/MaterialItemUtils.h"
#include "GUI/Models/Material/MaterialModelStore.h"
#include "GUI/utils/Helpers.h"

MaterialModel::MaterialModel(QObject* parent)
    : SessionModel(GUI::Session::XML::MaterialModelTag, parent)
{
    setObjectName(GUI::Session::XML::MaterialModelTag);
    if (GUI::Model::MaterialModelStore::materialModel() == nullptr)
        GUI::Model::MaterialModelStore::subscribe(this);
}

MaterialModel::~MaterialModel()
{
    if (GUI::Model::MaterialModelStore::materialModel() == this)
        GUI::Model::MaterialModelStore::unsubscribe(this);
}

MaterialModel* MaterialModel::createCopy(SessionItem* parent)
{
    MaterialModel* result = new MaterialModel();
    result->initFrom(this, parent);
    return result;
}

MaterialItem* MaterialModel::addRefractiveMaterial(const QString& name, double delta, double beta)
{
    auto materialItem = insertItem<MaterialItem>();
    materialItem->setItemName(name);
    materialItem->setColor(GUI::Model::MaterialItemUtils::suggestMaterialColor(name));
    materialItem->setRefractiveIndex(delta, beta);
    return materialItem;
}

MaterialItem* MaterialModel::addSLDMaterial(const QString& name, double sld, double abs_term)
{
    auto materialItem = insertItem<MaterialItem>();
    materialItem->setItemName(name);
    materialItem->setColor(GUI::Model::MaterialItemUtils::suggestMaterialColor(name));
    materialItem->setScatteringLengthDensity(complex_t(sld, abs_term));
    return materialItem;
}

MaterialItem* MaterialModel::materialFromName(const QString& name)
{
    for (auto materialItem : topItems<MaterialItem>())
        if (materialItem->itemName() == name)
            return materialItem;

    return nullptr;
}

MaterialItem* MaterialModel::materialFromIdentifier(const QString& identifier)
{
    for (auto materialItem : topItems<MaterialItem>())
        if (materialItem->identifier() == identifier)
            return materialItem;

    return nullptr;
}

MaterialItem* MaterialModel::cloneMaterial(MaterialItem* material)
{
    if (!material)
        return nullptr;

    auto clonedMaterial = copyItem(material, nullptr);
    clonedMaterial->setIdentifier(GUI::Helpers::createUuid());
    clonedMaterial->setItemName(material->itemName() + " (copy)");
    return clonedMaterial;
}

QVector<MaterialItem*> MaterialModel::materialItems() const
{
    return topItems<MaterialItem>();
}


MaterialItem* MaterialModel::findMaterialItem(const QString& identifier) const
{
    for (auto* m : materialItems())
        if (m->identifier() == identifier)
            return m;

    return nullptr;
}

void MaterialModel::removeMaterial(const QString& identifier)
{
    removeMaterial(materialFromIdentifier(identifier));
}

void MaterialModel::removeMaterial(MaterialItem* materialItem)
{
    if (materialItem != nullptr)
        if (const auto index = indexOfItem(materialItem); index.isValid())
            removeRows(index.row(), 1, index.parent());
}

void MaterialModel::readFrom(QXmlStreamReader* reader, MessageService* messageService /*= 0*/)
{
    // do not send added-notifications until completely read - otherwise partially
    // initialized items will be notified
    disconnect(this, &SessionModel::rowsInserted, this, &MaterialModel::onRowsChange);

    SessionModel::readFrom(reader, messageService);

    connect(this, &SessionModel::rowsInserted, this, &MaterialModel::onRowsChange);

    for (auto materialItem : materialItems()) {
        materialItem->mapper()->setOnAnyChildChange(
            [this, materialItem](SessionItem*) { materialChanged(materialItem); }, this);
    }
}

void MaterialModel::initFrom(SessionModel* model, SessionItem* parent)
{
    // since initFrom does not send materialChanged signals, this has to be done manually.

    QStringList identifiersOfChangedMaterials;
    auto* sourceModel = dynamic_cast<MaterialModel*>(model);
    for (auto destItem : materialItems()) {
        auto* srcItem = sourceModel->findMaterialItem(destItem->identifier());
        if (srcItem && (*srcItem != *destItem))
            identifiersOfChangedMaterials << destItem->identifier();
    }

    clear();
    SessionModel::initFrom(model, parent);

    for (auto identifier : identifiersOfChangedMaterials)
        emit materialChanged(findMaterialItem(identifier));
}

void MaterialModel::onRowsChange(const QModelIndex& parent, int, int)
{
    // valid parent means not a material (which is top level item) but something below
    if (parent.isValid())
        return;

    for (auto materialItem : materialItems()) {
        materialItem->mapper()->unsubscribe(this);

        materialItem->mapper()->setOnAnyChildChange(
            [this, materialItem](SessionItem*) { materialChanged(materialItem); }, this);
    }
}

bool MaterialModel::operator!=(const MaterialModel& other) const
{
    return !operator==(other);
}

bool MaterialModel::operator==(const MaterialModel& other) const
{
    const auto myItems = materialItems();
    const auto otherItems = other.materialItems();
    if (myItems.size() != otherItems.size())
        return false;

    for (int i = 0; i < myItems.size(); i++)
        if (*myItems[i] != *otherItems[i])
            return false;

    return true;
}
