//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Models/Sample/SampleValidator.cpp
//! @brief     Implements class SampleValidator
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Models/Sample/SampleValidator.h"
#include "GUI/Items/LayerItem.h"
#include "GUI/Items/MultiLayerItem.h"
#include "GUI/Items/ParticleCompositionItem.h"
#include "GUI/Items/ParticleCoreShellItem.h"
#include "GUI/Items/ParticleLayoutItem.h"

#include <boost/polymorphic_cast.hpp>
using boost::polymorphic_cast;
using boost::polymorphic_downcast;

SampleValidator::SampleValidator() : m_valid_sample(true) {}

void SampleValidator::initValidator()
{
    m_validation_message.clear();
    m_valid_sample = true;
}

void SampleValidator::iterateItems(const SessionItem* parentItem)
{
    for (const SessionItem* child : parentItem->children()) {
        validateItem(child);
        iterateItems(child);
    }
}

void SampleValidator::validateItem(const SessionItem* item)
{
    if (!item)
        return;

    QString diagnosis;

    if (item->hasModelType<MultiLayerItem>()) {
        diagnosis = validateMultiLayerItem(polymorphic_downcast<const MultiLayerItem*>(item));
    } else if (item->hasModelType<ParticleLayoutItem>()) {
        diagnosis =
            validateParticleLayoutItem(polymorphic_downcast<const ParticleLayoutItem*>(item));
    } else if (item->hasModelType<ParticleCoreShellItem>()) {
        diagnosis =
            validateParticleCoreShellItem(polymorphic_cast<const ParticleCoreShellItem*>(item));
    } else if (item->hasModelType<ParticleCompositionItem>()) {
        diagnosis =
            validateParticleCompositionItem(polymorphic_cast<const ParticleCompositionItem*>(item));
    }
    if (!diagnosis.isEmpty()) {
        m_valid_sample = false;
        m_validation_message += QString("* ") + diagnosis + "\n";
    }
}

QString SampleValidator::validateMultiLayerItem(const MultiLayerItem* item)
{
    QString result;

    QVector<LayerItem*> layers = item->layers();

    if (layers.isEmpty()) {
        result = "MultiLayer should contain at least one layer.";
    } else if (layers.size() == 1) {
        if (layers.front()->layouts().isEmpty()) {
            result = "The single layer in your MultiLayer should contain ParticleLayout.";
        }
    }
    return result;
}

QString SampleValidator::validateParticleLayoutItem(const ParticleLayoutItem* item)
{
    QString result;

    QVector<ItemWithParticles*> particles = item->particles();
    if (particles.isEmpty())
        result = "ParticleLayout doesn't contain any particles.";

    return result;
}

QString SampleValidator::validateParticleCoreShellItem(const ParticleCoreShellItem* item)
{
    QString result;

    const ParticleItem* core = item->core();
    const ParticleItem* shell = item->shell();

    if (core == nullptr || shell == nullptr)
        result = "ParticleCoreShell doesn't have either core or shell defined.";

    return result;
}

QString SampleValidator::validateParticleCompositionItem(const ParticleCompositionItem* item)
{
    QString result;
    if (item->particles().isEmpty())
        result = "ParticleComposition doesn't have any particles.";

    return result;
}

bool SampleValidator::isValidMultiLayer(const MultiLayerItem* multilayer)
{
    initValidator();

    validateItem(multilayer);
    iterateItems(multilayer);

    if (!m_valid_sample) {
        m_validation_message =
            "Can't setup DWBA simulation for given MultiLayer.\n" + m_validation_message;
    }
    return m_valid_sample;
}
