//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/DepthProbeInstrumentEditor.cpp
//! @brief     Implements class DepthProbeInstrumentEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/DepthProbeInstrumentEditor.h"
#include "GUI/CommonWidgets/StyleUtils.h"
#include "GUI/Items/BeamWavelengthItem.h"
#include "GUI/Items/InstrumentItems.h"
#include "GUI/Views/InstrumentWidgets/DistributionEditor.h"
#include "GUI/Views/InstrumentWidgets/InclinationAnglesEditor.h"
#include "GUI/Views/InstrumentWidgets/PolarizationAnalysisEditor.h"
#include "GUI/Views/InstrumentWidgets/SphericalAxisEditor.h"

#include <QGridLayout>
#include <QVBoxLayout>

DepthProbeInstrumentEditor::DepthProbeInstrumentEditor(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f)
{
    QVBoxLayout* layout = new QVBoxLayout(this);

    QGridLayout* grid = new QGridLayout();
    grid->setColumnStretch(0, 1);
    grid->setColumnStretch(1, 1);
    grid->setColumnStretch(2, 1);

    // valid input range is ]0,inf[
    MeanConfig wave_cfg{boost::numeric::interval<double>(std::numeric_limits<double>::min(),
                                                         std::numeric_limits<double>::max()),
                        4, true};
    m_wavelengthEditor =
        new DistributionEditor("Wavelength [nm]", wave_cfg, Distributions::Symmetric, this);
    grid->addWidget(m_wavelengthEditor, 1, 0);

    m_inclinationEditor = new InclinationAnglesEditor(this);
    grid->addWidget(m_inclinationEditor, 1, 1);

    m_depthAxisEditor = new SphericalAxisEditor("Depth axis [nm]", this);
    grid->addWidget(m_depthAxisEditor, 1, 2);

    layout->addWidget(GUI::Utils::Style::createDetailsWidget(grid, "Parameters"));

    m_polpairAnalysisEditor = new PolarizationAnalysisEditor(this);
    layout->addWidget(GUI::Utils::Style::createDetailsWidget(
        m_polpairAnalysisEditor, "Polarization analysis", /*expanded*/ false));

    layout->addStretch();
}

void DepthProbeInstrumentEditor::setItem(DepthProbeInstrumentItem* item)
{
    if (item) {
        m_wavelengthEditor->setItem(item->beamItem()->wavelengthItem());
        m_inclinationEditor->setItem(item->beamItem()->inclinationAngleItem());
        m_depthAxisEditor->setItem(item->zAxis());
    } else {
        m_wavelengthEditor->clearItem();
        m_inclinationEditor->clearItem();
        m_depthAxisEditor->clearItem();
    }
    m_polpairAnalysisEditor->setItem(item);
}
