//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/DistributionEditor.h
//! @brief     A widget for selecting a distribution and setting its parameters
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONEDITOR_H
#define BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONEDITOR_H

#include <QWidget>

class QComboBox;
class QStackedLayout;

class DistributionForm;
class ItemWithDistribution;

#include <boost/numeric/interval.hpp>

#include <map>
#include <optional>

/// configuration to control how the user can enter a mean value
struct MeanConfig {
    boost::numeric::interval<double> range; // range for the mean value
    int decimals;                           // number of decimals shown in the input
    bool scientific;                        // whether to use a scientific number notation
};

/// enum used to configure which distributions should be available in the selector
enum class Distributions { All, Symmetric };

/// Widget for selecting a distribution (combo box) and input of the corresponding values
/// with respect to the given distribution (e.g. mean and deviation for gauss distribution)

class DistributionSelector : public QWidget {
    Q_OBJECT

public:
    /// \param mean_config controls how the mean value is shown and can be input.
    ///                    If this parameter is \c nullopt then the mean value can not be input
    ///                    and only symmetric distributions are provided
    /// \pre ! mean_config && distributions == Distributions::Symmetric
    ///      (i.e. the combination of all distributions without mean input is currently not
    ///       supported)
    DistributionSelector(const std::optional<MeanConfig>& mean_config, Distributions distributions,
                         QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(ItemWithDistribution* item);
    ItemWithDistribution* item() const;
    const std::optional<MeanConfig>& meanConfig() const;
    Distributions distributions() const;
    void clearItem();
    void refresh();

signals:
    void distributionChanged();

private slots:
    void onDistributionSelected(int index);

private:
    QComboBox* m_combo;
    QStackedLayout* m_stack;
    std::map<QString, DistributionForm*> m_forms;
    ItemWithDistribution* m_item;
    std::optional<MeanConfig> m_meanConfig;
    Distributions m_distributions;
};


/// DistributionSelector as above but contained in a GroupBox with a title and a button
/// to open the distribution dialog

class DistributionEditor : public QWidget {
    Q_OBJECT

public:
    DistributionEditor(const QString& title, const std::optional<MeanConfig>& mean_config,
                       Distributions distributions, QWidget* parent = nullptr,
                       Qt::WindowFlags f = Qt::WindowFlags());
    void setItem(ItemWithDistribution* item);
    void clearItem();

signals:
    void distributionChanged();

private slots:
    void showDialog();

private:
    QString m_title;
    DistributionSelector* m_selector;
};

/// utility function to show the EditDistributionDialog for a selector
void showEditDistributionDialog(DistributionSelector* selector, const QString& title);

#endif // BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONEDITOR_H
