//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/DistributionForms.h
//! @brief     The forms for the various distributions used by DistributionSelector
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONFORMS_H
#define BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONFORMS_H

#include <QWidget>

#include <optional>
#include <variant>

struct MeanConfig;
class ItemWithDistribution;
class DistributionCosineItem;
class DistributionGateItem;
class DistributionGaussianItem;
class DistributionLogNormalItem;
class DistributionLorentzItem;
class DistributionNoneItem;
class DistributionTrapezoidItem;

class QSpinBox;
class QDoubleSpinBox;
class ScientificSpinBox;

class DistributionForm : public QWidget {
    Q_OBJECT

public:
    /// Set type if not already the matching one and display parameters of distribution
    virtual void setupDistribution(ItemWithDistribution* item) = 0;

signals:
    void distributionChanged();

protected:
    DistributionForm(QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
};


class CosineDistributionForm : public DistributionForm {
    Q_OBJECT

public:
    CosineDistributionForm(const std::optional<MeanConfig>& mean_config, QWidget* parent = nullptr,
                           Qt::WindowFlags f = Qt::WindowFlags());
    void setupDistribution(ItemWithDistribution* item) override;

private slots:
    void onMeanChanged(double value);
    void onSigmaChanged(double value);
    void onSamplesChanged(int value);
    void onSigmaFactorChanged(double value);

private:
    std::variant<QDoubleSpinBox*, ScientificSpinBox*> m_meanSpinBox;
    QDoubleSpinBox* m_sigmaSpinBox;
    QSpinBox* m_samplesSpinBox;
    QDoubleSpinBox* m_sigmaFactorSpinBox;
    DistributionCosineItem* m_item;
};

class GateDistributionForm : public DistributionForm {
    Q_OBJECT

public:
    GateDistributionForm(QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setupDistribution(ItemWithDistribution* item) override;

private slots:
    void onMinimumChanged(double value);
    void onMaximumChanged(double value);
    void onSamplesChanged(int value);

private:
    QDoubleSpinBox* m_minimumSpinBox;
    QDoubleSpinBox* m_maximumSpinBox;
    QSpinBox* m_samplesSpinBox;
    DistributionGateItem* m_item;
};

class GaussDistributionForm : public DistributionForm {
    Q_OBJECT

public:
    GaussDistributionForm(const std::optional<MeanConfig>& mean_config, QWidget* parent = nullptr,
                          Qt::WindowFlags f = Qt::WindowFlags());
    void setupDistribution(ItemWithDistribution* item) override;

private slots:
    void onMeanChanged(double value);
    void onDeviationChanged(double value);
    void onSamplesChanged(int value);
    void onSigmaFactorChanged(double value);

private:
    std::variant<QDoubleSpinBox*, ScientificSpinBox*> m_meanSpinBox;
    QDoubleSpinBox* m_stdDevSpinBox;
    QSpinBox* m_samplesSpinBox;
    QDoubleSpinBox* m_sigmaFactorSpinBox;
    DistributionGaussianItem* m_item;
};


class LogNormalDistributionForm : public DistributionForm {
    Q_OBJECT

public:
    LogNormalDistributionForm(QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setupDistribution(ItemWithDistribution* item) override;

private slots:
    void onMedianChanged(double value);
    void onScaleChanged(double value);
    void onSamplesChanged(int value);
    void onSigmaFactorChanged(double value);

private:
    QDoubleSpinBox* m_medianSpinBox;
    QDoubleSpinBox* m_scaleSpinBox;
    QSpinBox* m_samplesSpinBox;
    QDoubleSpinBox* m_sigmaFactorSpinBox;
    DistributionLogNormalItem* m_item;
};

class LorentzDistributionForm : public DistributionForm {
    Q_OBJECT

public:
    LorentzDistributionForm(const std::optional<MeanConfig>& mean_config, QWidget* parent = nullptr,
                            Qt::WindowFlags f = Qt::WindowFlags());
    void setupDistribution(ItemWithDistribution* item) override;

private slots:
    void onMeanChanged(double value);
    void onHWHMChanged(double value);
    void onSamplesChanged(int value);
    void onSigmaFactorChanged(double value);

private:
    std::variant<QDoubleSpinBox*, ScientificSpinBox*> m_meanSpinBox;
    QDoubleSpinBox* m_hwhmSpinBox;
    QSpinBox* m_samplesSpinBox;
    QDoubleSpinBox* m_sigmaFactorSpinBox;
    DistributionLorentzItem* m_item;
};

class NoneDistributionForm : public DistributionForm {
    Q_OBJECT

public:
    NoneDistributionForm(const std::optional<MeanConfig>& mean_config, QWidget* parent = nullptr,
                         Qt::WindowFlags f = Qt::WindowFlags());
    void setupDistribution(ItemWithDistribution* item) override;

private slots:
    void onValueChanged(double value);

private:
    std::variant<QDoubleSpinBox*, ScientificSpinBox*> m_spinBox;
    DistributionNoneItem* m_item;
};

class TrapezoidDistributionForm : public DistributionForm {
    Q_OBJECT

public:
    TrapezoidDistributionForm(QWidget* parent = nullptr, Qt::WindowFlags f = Qt::WindowFlags());
    void setupDistribution(ItemWithDistribution* item) override;

private slots:
    void onCenterChanged(double value);
    void onLeftWidthChanged(double value);
    void onMiddleWidthChanged(double value);
    void onRightWidthChanged(double value);
    void onSamplesChanged(int value);

private:
    QDoubleSpinBox* m_centerSpinBox;
    QDoubleSpinBox* m_leftWidthSpinBox;
    QDoubleSpinBox* m_middleWidthSpinBox;
    QDoubleSpinBox* m_rightWidthSpinBox;
    QSpinBox* m_samplesSpinBox;
    DistributionTrapezoidItem* m_item;
};

#endif // BORNAGAIN_GUI_VIEWS_INSTRUMENTWIDGETS_DISTRIBUTIONFORMS_H
