//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/InclinationAnglesEditor.cpp
//! @brief     Defines class InclinationAnglesEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/InclinationAnglesEditor.h"
#include "GUI/Items/GroupItem.h"
#include "GUI/Items/PointwiseAxisItem.h"
#include "GUI/Items/SpecularBeamInclinationItem.h"
#include "GUI/Views/InstrumentWidgets/DistributionEditor.h"
#include "GUI/Views/InstrumentWidgets/SphericalAxisEditor.h"
#include "GUI/Views/PropertyEditor/GroupInfoBox.h"

#include <QComboBox>
#include <QFormLayout>

//==================================================================================================
// InclinationAnglesEditor
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// private static members
//--------------------------------------------------------------------------------------------------

const QString InclinationAnglesEditor::m_title = "Inclination angles [deg]";

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

InclinationAnglesEditor::InclinationAnglesEditor(QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f), m_item(nullptr)
{
    QVBoxLayout* layout = new QVBoxLayout(this);

    GroupInfoBox* gbox = new GroupInfoBox(m_title, this);
    QFormLayout* gform = new QFormLayout(gbox);
    connect(gbox, &GroupInfoBox::clicked, this, &InclinationAnglesEditor::showDialog);
    layout->addWidget(gbox);

    m_distributionEditor = new DistributionSelector(std::nullopt, Distributions::Symmetric, gbox);
    gform->addRow(m_distributionEditor);

    m_typeComboBox = new QComboBox(gbox);
    m_typeComboBox->addItem("Uniform axis", BasicAxisItem::M_TYPE);
    m_typeComboBox->addItem("Non-uniform axis", PointwiseAxisItem::M_TYPE);
    gform->addRow("Axis type:", m_typeComboBox);
    connect(m_typeComboBox, qOverload<int>(&QComboBox::currentIndexChanged), this,
            &InclinationAnglesEditor::onAxisTypeSelected);

    m_form = new SphericalAxisForm(gform, gbox);
}

void InclinationAnglesEditor::setItem(SpecularBeamInclinationItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        m_distributionEditor->setItem(m_item);
        int idx = m_typeComboBox->findData(m_item->alphaAxisItem()->currentType());
        ASSERT(idx != -1);
        if (idx != m_typeComboBox->currentIndex())
            m_typeComboBox->setCurrentIndex(idx);
        else
            onAxisTypeSelected(idx); // trigger update manually
        m_typeComboBox->setEnabled(m_item->alphaAxisItem()->isEnabled());
    } else {
        m_distributionEditor->clearItem();
        m_typeComboBox->setCurrentIndex(0);
        m_form->clearItem();
    }
}

void InclinationAnglesEditor::clearItem()
{
    setItem(nullptr);
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void InclinationAnglesEditor::onAxisTypeSelected(int index)
{
    if (m_item) {
        QString axis_type = m_typeComboBox->itemData(index).toString();
        if (m_item->alphaAxisItem()->currentType() != axis_type)
            m_item->alphaAxisItem()->setCurrentType(axis_type);
        m_form->setItem(m_item->alphaAxis());
        m_form->setEnabled(axis_type != PointwiseAxisItem::M_TYPE);
    }
}

void InclinationAnglesEditor::showDialog()
{
    showEditDistributionDialog(m_distributionEditor, m_title);
}
