//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/InstrumentWidgets/SphericalAxisEditor.cpp
//! @brief     Implement class SphericalAxisEditor
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/InstrumentWidgets/SphericalAxisEditor.h"
#include "GUI/Items/AxesItems.h"

#include <QDoubleSpinBox>
#include <QFormLayout>
#include <QGroupBox>
#include <QSpinBox>


//==================================================================================================
// SphericalAxisForm
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

SphericalAxisForm::SphericalAxisForm(QFormLayout* form, QWidget* parent)
    : QObject(parent), m_item(nullptr)
{
    m_nbinsSpinBox = new QSpinBox(parent);
    m_nbinsSpinBox->setRange(1, 65536);
    connect(m_nbinsSpinBox, qOverload<int>(&QSpinBox::valueChanged), this,
            &SphericalAxisForm::onNbinsValueChanged);
    form->addRow("Nbins:", m_nbinsSpinBox);

    m_minimumSpinBox = new QDoubleSpinBox(parent);
    m_minimumSpinBox->setRange(std::numeric_limits<double>::lowest(),
                               std::numeric_limits<double>::max());
    m_minimumSpinBox->setDecimals(3);
    m_minimumSpinBox->setSingleStep(0.01);
    connect(m_minimumSpinBox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &SphericalAxisForm::onMinimumValueChanged);
    form->addRow("Minimum [deg]:", m_minimumSpinBox);

    m_maximumSpinBox = new QDoubleSpinBox(parent);
    m_maximumSpinBox->setRange(std::numeric_limits<double>::lowest(),
                               std::numeric_limits<double>::max());
    m_maximumSpinBox->setDecimals(3);
    m_maximumSpinBox->setSingleStep(0.01);
    connect(m_maximumSpinBox, qOverload<double>(&QDoubleSpinBox::valueChanged), this,
            &SphericalAxisForm::onMaximumValueChanged);
    form->addRow("Maximum [deg]:", m_maximumSpinBox);
}

void SphericalAxisForm::setItem(BasicAxisItem* item)
{
    m_item = item;
    setEnabled(m_item);
    if (m_item) {
        m_nbinsSpinBox->setValue(m_item->binCount());
        m_minimumSpinBox->setValue(m_item->lowerBound());
        m_maximumSpinBox->setValue(m_item->upperBound());
    } else {
        m_nbinsSpinBox->setValue(1);
        m_minimumSpinBox->setValue(0.0);
        m_maximumSpinBox->setValue(0.0);
    }
}

void SphericalAxisForm::clearItem()
{
    setItem(nullptr);
}

void SphericalAxisForm::setEnabled(bool enabled)
{
    m_nbinsSpinBox->setEnabled(enabled);
    m_minimumSpinBox->setEnabled(enabled);
    m_maximumSpinBox->setEnabled(enabled);
}

//--------------------------------------------------------------------------------------------------
// private slots
//--------------------------------------------------------------------------------------------------

void SphericalAxisForm::onNbinsValueChanged(int value)
{
    if (m_item)
        m_item->setBinCount(value);
}

void SphericalAxisForm::onMinimumValueChanged(double value)
{
    if (m_item) {
        m_item->setLowerBound(value);
        if (m_item->upperBound() < value)
            m_maximumSpinBox->setValue(value);
    }
}

void SphericalAxisForm::onMaximumValueChanged(double value)
{
    if (m_item) {
        m_item->setUpperBound(value);
        if (m_item->lowerBound() > value)
            m_minimumSpinBox->setValue(value);
    }
}

//==================================================================================================
// SphericalAxisEditor
//==================================================================================================

//--------------------------------------------------------------------------------------------------
// public member functions
//--------------------------------------------------------------------------------------------------

SphericalAxisEditor::SphericalAxisEditor(const QString& title, QWidget* parent, Qt::WindowFlags f)
    : QWidget(parent, f)
{
    QVBoxLayout* layout = new QVBoxLayout(this);
    QGroupBox* gbox = new QGroupBox(title, this);
    QFormLayout* form = new QFormLayout(gbox);
    layout->addWidget(gbox);

    m_form = new SphericalAxisForm(form, gbox);
}

void SphericalAxisEditor::setItem(BasicAxisItem* item)
{
    m_form->setItem(item);
}

void SphericalAxisEditor::clearItem()
{
    m_form->clearItem();
}
