//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/JobWidgets/JobResultsPresenter.cpp
//! @brief     Implements class JobResultsPresenter
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/JobWidgets/JobResultsPresenter.h"
#include "GUI/Items/InstrumentItems.h"
#include "GUI/Items/JobItem.h"
#include "GUI/Views/IntensityDataWidgets/IntensityDataWidget.h"
#include "GUI/Views/JobWidgets/FitComparisonWidget.h"
#include "GUI/Views/JobWidgets/FitComparisonWidget1D.h"
#include "GUI/Views/JobWidgets/JobViewActivities.h"
#include "GUI/Views/ProjectionsWidgets/IntensityDataProjectionsWidget.h"
#include "GUI/Views/SpecularDataWidgets/SpecularDataWidget.h"

JobResultsPresenter::JobResultsPresenter(QWidget* parent) : ItemComboWidget(parent)
{
    setWindowTitle("Job OutputData");
    setObjectName("JobResultsPresenter");

    registerWidget("Color Map", create_new<IntensityDataWidget>);
    registerWidget("Projections", create_new<IntensityDataProjectionsWidget>);
    registerWidget("Fit 1D Data", create_new<FitComparisonWidget1D>);
    registerWidget("Fit 2D Data", create_new<FitComparisonWidget>);
    registerWidget("Reflectometry", create_new<SpecularDataWidget>);

    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    setMinimumSize(600, 600);
}

QString JobResultsPresenter::itemPresentation() const
{
    if (!currentItem())
        return {};

    const QString value = dynamic_cast<const JobItem*>(currentItem())->presentationType();
    return value.isEmpty() ? selectedPresentation() : value;
}

void JobResultsPresenter::setItem(SessionItem* item)
{
    if (currentItem())
        currentItem()->mapper()->unsubscribe(this);
    if (item)
        item->mapper()->setOnPropertyChange(
            std::bind(&JobResultsPresenter::notifyJobPropertyChanged, this, std::placeholders::_1));
    ItemComboWidget::setItem(item);
}

void JobResultsPresenter::setPresentation(const QString& presentationType)
{
    ItemComboWidget::setPresentation(presentationType);
    JobItem* item = dynamic_cast<JobItem*>(currentItem());
    if (item && !presentationType.isEmpty())
        item->setPresentationType(presentationType);
}

void JobResultsPresenter::setPresentation(JobViewActivity activity)
{
    m_activity = activity;
    QString presentation;
    JobItem* jobItem = dynamic_cast<JobItem*>(currentItem());
    if (jobItem && !jobItem->isRunning()) {
        switch (activity) {
        case JobViewActivity::Fitting:
            presentation = fitPresentationOfInstrument(jobItem->instrumentItem());
            break;

        case JobViewActivity::RealTime:
        case JobViewActivity::JobView:
            presentation = defaultPresentationOfInstrument(jobItem->instrumentItem());
            break;
        }
    }

    setPresentation(presentation);
}

//! Returns list of presentation types, available for given item. JobItem with fitting abilities
//! is valid for IntensityDataWidget and FitComparisonWidget.

QStringList JobResultsPresenter::activePresentationList(SessionItem* item)
{
    JobItem* jobItem = dynamic_cast<JobItem*>(item);
    if (!jobItem || jobItem->isRunning())
        return {};

    ASSERT(jobItem->instrumentItem() != nullptr);

    auto result = defaultActivePresentationsOfInstrument(jobItem->instrumentItem());

    if (jobItem->isValidForFitting())
        result << fitPresentationOfInstrument(jobItem->instrumentItem());

    return result;
}

QStringList JobResultsPresenter::presentationList(SessionItem* item)
{
    JobItem* jobItem = dynamic_cast<JobItem*>(item);
    if (!jobItem || jobItem->isRunning())
        return {};

    ASSERT(jobItem->instrumentItem() != nullptr);

    auto result = defaultActivePresentationsOfInstrument(jobItem->instrumentItem());
    auto addon = fitPresentationOfInstrument(jobItem->instrumentItem());
    if (!addon.isEmpty())
        result << addon;

    return result;
}

QString JobResultsPresenter::defaultPresentationOfInstrument(InstrumentItem* instrumentItem)
{
    if (instrumentItem->is<SpecularInstrumentItem>())
        return "Reflectometry";

    return "Color Map";
}

QStringList
JobResultsPresenter::defaultActivePresentationsOfInstrument(InstrumentItem* instrumentItem)
{
    if (instrumentItem->is<SpecularInstrumentItem>())
        return {"Reflectometry"};

    return {"Color Map", "Projections"};
}

QString JobResultsPresenter::fitPresentationOfInstrument(InstrumentItem* instrumentItem)
{
    if (instrumentItem->is<SpecularInstrumentItem>())
        return "Fit 1D Data";

    return "Fit 2D Data";
}

void JobResultsPresenter::notifyJobPropertyChanged(const QString& property)
{
    if (JobItem::isStatusPropertyName(property))
        setPresentation(m_activity);
}
