//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/MaskWidgets/IShape2DView.cpp
//! @brief     Implements interface IShape2DView.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/MaskWidgets/IShape2DView.h"
#include "GUI/Items/MaskItems.h"
#include "GUI/Views/MaskWidgets/ISceneAdaptor.h"
#include <QAction>
#include <QGraphicsScene>
#include <QGraphicsSceneContextMenuEvent>
#include <QMenu>
#include <QPainter>

namespace {
bool itemMaskValue(const SessionItem* item)
{
    if (const MaskItem* maskItem = dynamic_cast<const MaskItem*>(item))
        return maskItem->maskValue();
    return false;
}
} // namespace

IShape2DView::IShape2DView(SessionItem* item)
    : m_adaptor(nullptr), m_block_on_property_change(false)
{
    connect(this, &IShape2DView::xChanged, this, &IShape2DView::onChangedX);
    connect(this, &IShape2DView::yChanged, this, &IShape2DView::onChangedY);

    if (item) {
        item->mapper()->setOnPropertyChange(
            [this](const QString& name) { onItemPropertyChange(name); }, this);

        item->mapper()->setOnItemDestroy([this](SessionItem*) { onItemDestroyed(); }, this);
    }
}

QRectF IShape2DView::boundingRect() const
{
    return m_bounding_rect;
}

void IShape2DView::setSceneAdaptor(const ISceneAdaptor* adaptor)
{
    ASSERT(adaptor);

    if (m_adaptor != adaptor) {
        if (m_adaptor)
            disconnect(m_adaptor, &ISceneAdaptor::update_request, this, &IShape2DView::update_view);

        m_adaptor = adaptor;
        connect(m_adaptor, &ISceneAdaptor::update_request, this, &IShape2DView::update_view,
                Qt::UniqueConnection);
        update_view();
    }
}

void IShape2DView::paint(QPainter* painter, const QStyleOptionGraphicsItem*, QWidget*)
{
    const bool isMasked = itemMaskValue(parameterizedItem());
    painter->setBrush(MaskEditorHelper::getMaskBrush(isMasked));
    painter->setPen(MaskEditorHelper::getMaskPen(isMasked));
    painter->setRenderHints(QPainter::Antialiasing);
    painter->drawPath(maskedShape());
}

QPainterPath IShape2DView::maskedShape() const
{
    static const QSet<MaskEditorHelper::EViewTypes> relevantMaskTypes = {
        MaskEditorHelper::RECTANGLE, MaskEditorHelper::POLYGON, MaskEditorHelper::VERTICALLINE,
        MaskEditorHelper::HORIZONTALLINE, MaskEditorHelper::ELLIPSE};

    QPainterPath resultingShape = mapToScene(shape());
    for (auto item : scene()->items()) {
        const auto maskItem = dynamic_cast<IShape2DView*>(item);
        if (!maskItem
            || !relevantMaskTypes.contains((MaskEditorHelper::EViewTypes)maskItem->type()))
            continue;

        const bool isMaskingItem = itemMaskValue(maskItem->parameterizedItem());
        const bool isOnTop = maskItem->zValue() > zValue();
        if (isMaskingItem || !isOnTop)
            continue;

        const auto maskItemShape = maskItem->mapToScene(maskItem->shape());
        if (!maskItemShape.intersects(resultingShape))
            continue;

        // Item lays on top and is non-masking -> subtract the
        // path of the item
        resultingShape = resultingShape.subtracted(maskItemShape);
    }

    return mapFromScene(resultingShape);
}

qreal IShape2DView::toSceneX(qreal value) const
{
    return m_adaptor ? m_adaptor->toSceneX(value) : value;
}

qreal IShape2DView::toSceneY(qreal value) const
{
    return m_adaptor ? m_adaptor->toSceneY(value) : value;
}

qreal IShape2DView::fromSceneX(qreal value) const
{
    return m_adaptor ? m_adaptor->fromSceneX(value) : value;
}

qreal IShape2DView::fromSceneY(qreal value) const
{
    return m_adaptor ? m_adaptor->fromSceneY(value) : value;
}

void IShape2DView::addView(IShape2DView* childView, int /* row */)
{
    if (!childItems().contains(childView))
        childView->setParentItem(this);
}

void IShape2DView::setBlockOnProperty(bool value)
{
    m_block_on_property_change = value;
}

bool IShape2DView::blockOnProperty() const
{
    return m_block_on_property_change;
}

void IShape2DView::disconnectFromItem(SessionItem* item)
{
    if (item)
        item->mapper()->unsubscribe(this);
}

void IShape2DView::onItemPropertyChange(const QString& propertyName)
{
    if (m_block_on_property_change)
        return;

    m_block_on_property_change = true;

    bool schedule_update = false;
    if (MaskItem::isMaskValuePropertyName(propertyName)) {
        schedule_update = true;
    } else if (MaskItem::isIsVisiblePropertyName(propertyName)) {
        bool visible = false;
        if (MaskItem* maskItem = dynamic_cast<MaskItem*>(parameterizedItem()))
            visible = maskItem->isVisibleValue();
        this->setVisible(visible);
        schedule_update = true;
    }

    onPropertyChange(propertyName);

    if (schedule_update)
        update();

    m_block_on_property_change = false;
}
