//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/SampleDesigner/ParticleCompositionForm.cpp
//! @brief     Implements class ParticleCompositionForm
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/SampleDesigner/ParticleCompositionForm.h"
#include "GUI/Items/ParticleCompositionItem.h"
#include "GUI/Models/Types/VectorDescriptor.h"
#include "GUI/Views/SampleDesigner/FormLayouter.h"
#include "GUI/Views/SampleDesigner/GroupBoxCollapser.h"
#include "GUI/Views/SampleDesigner/LayerEditorUtils.h"
#include "GUI/utils/ActionFactory.h"

#include <QAction>
#include <QPushButton>

ParticleCompositionForm::ParticleCompositionForm(QWidget* parent,
                                                 ParticleCompositionItem* compositionItem,
                                                 SampleEditorController* ec, bool allowRemove)
    : QGroupBox(parent), m_compositionItem(compositionItem), m_ec(ec)
{
    setTitle("Composition of particles");

    FormLayouter layouter(this, ec);
    layouter.setContentsMargins(30, 6, 0, 0);
    layouter.addVector(compositionItem->positionVector(), false);
    layouter.addSelection(compositionItem->rotationMethod());
    layouter.addValue(compositionItem->abundance());

    for (auto particle : compositionItem->particles())
        layouter.addRow(LayerEditorUtils::createWidgetForItemWithParticles(this, particle, ec));

    auto* btn = LayerEditorUtils::createAddParticleButton(
        this, [=](const QString& cls) { ec->addParticle(compositionItem, cls); });

    m_structureEditingWidgets << btn;
    layouter.addStructureEditingRow(btn);

    auto* collapser = GroupBoxCollapser::installIntoGroupBox(this);

    if (allowRemove) {
        m_removeAction = ActionFactory::createRemoveAction(
            this, "particle composition", [=] { ec->removeParticle(compositionItem); });
        collapser->addAction(m_removeAction);
    }

    m_layout = layouter.layout();
}

void ParticleCompositionForm::enableStructureEditing(bool b)
{
    if (m_removeAction)
        m_removeAction->setVisible(b);
    for (auto w : m_structureEditingWidgets)
        w->setVisible(b);
}

ParticleCompositionItem* ParticleCompositionForm::compositionItem() const
{
    return m_compositionItem;
}

void ParticleCompositionForm::onParticleAdded(SessionItem* newItem)
{
    if (auto p = dynamic_cast<ItemWithParticles*>(newItem)) {
        int index = m_compositionItem->particles().indexOf(p);
        const int rowInLayout = m_layout->rowCount() - 1
                                - (m_compositionItem->particles().size() - 1) + index; // -1: btn

        m_layout->insertRow(rowInLayout,
                            LayerEditorUtils::createWidgetForItemWithParticles(this, p, m_ec));
    }
}

void ParticleCompositionForm::onAboutToRemoveParticle(ItemWithParticles* item)
{
    int index = m_compositionItem->particles().indexOf(item);
    const int rowInLayout =
        m_layout->rowCount() - m_compositionItem->particles().size() - 1 + index; // -1: btn

    m_layout->removeRow(rowInLayout);
}
