//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/SampleDesigner/SampleToolBox.cpp
//! @brief     Implements class SampleToolBox
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/Views/SampleDesigner/SampleToolBox.h"
#include "GUI/Items/ItemCatalog.h"
#include "GUI/Items/LayerItem.h"
#include "GUI/Items/MultiLayerItem.h"
#include "GUI/Items/ParticleLayoutItem.h"
#include "GUI/Items/TransformationItem.h"
#include "GUI/Models/Sample/GUIExamplesFactory.h"
#include "GUI/Views/SampleDesigner/DesignerMimeData.h"
#include "GUI/Views/SampleDesigner/SampleToolBoxTreeWidget.h"
#include <QApplication>
#include <QFocusEvent>
#include <QIcon>
#include <QLineEdit>
#include <QToolBar>
#include <QVBoxLayout>

namespace {

class FilterEdit : public QLineEdit {
public:
    explicit FilterEdit(QWidget* parent = 0) : QLineEdit(parent) { setFocusPolicy(Qt::NoFocus); }

protected:
    virtual void mousePressEvent(QMouseEvent* event)
    {
        if (!hasFocus()) // Explicitly focus on click.
            setFocus(Qt::OtherFocusReason);
        QLineEdit::mousePressEvent(event);
    }

    virtual void focusInEvent(QFocusEvent* e)
    {
        // Refuse the focus if the mouse it outside. In addition to the mouse
        // press logic, this prevents a re-focussing which occurs once
        // we actually had focus
        const Qt::FocusReason reason = e->reason();
        if (reason == Qt::ActiveWindowFocusReason || reason == Qt::PopupFocusReason) {
            const QPoint mousePos = mapFromGlobal(QCursor::pos());
            const bool refuse = !geometry().contains(mousePos);
            if (refuse) {
                e->ignore();
                return;
            }
        }
        QLineEdit::focusInEvent(e);
    }
};

} // namespace


SampleToolBox::SampleToolBox(QWidget* parent) : QWidget(parent), m_view(new SampleToolBoxTreeWidget)
{
    setObjectName(QLatin1String("SampleToolBox"));
    setWindowTitle("Items Toolbox");

    // Prevent the filter from grabbing focus since Our view has Qt::NoFocus
    QToolBar* toolBar = new QToolBar(this);
    QLineEdit* filterEdit = new FilterEdit(toolBar);
    filterEdit->setPlaceholderText("Filter");
    filterEdit->setClearButtonEnabled(true);
    connect(filterEdit, &QLineEdit::textChanged, m_view, &SampleToolBoxTreeWidget::filter);
    toolBar->addWidget(filterEdit);

    QVBoxLayout* layout = new QVBoxLayout(this);
    layout->setMargin(0);
    layout->setSpacing(0);
    layout->addWidget(toolBar);
    layout->addWidget(m_view);
    setLayout(layout);

    connect(m_view, &SampleToolBoxTreeWidget::pressed, this, &SampleToolBox::handleMousePress);

    fill();
}

QSize SampleToolBox::minimumSizeHint() const
{
    return QSize(300, 300);
}

void SampleToolBox::handleMousePress(const QString& /*title*/, const QString& className,
                                     const QPoint&)
{
    if (QApplication::mouseButtons() != Qt::LeftButton)
        return;

    DesignerMimeData::execDrag(className, this);
}

void SampleToolBox::fill()
{
    const auto icon = [](const QString& iconName) {
        return QIcon(":/SampleDesignerToolbox/images/" + iconName);
    };

    m_view->addGroup("Sample");
    m_view->addEntry(MultiLayerItem::M_TYPE, icon("MultiLayer.png"), "Multi Layer",
                     "A multilayer to hold stack of layers");

    m_view->addGroup("Layers");
    m_view->addEntry(LayerItem::M_TYPE, icon("Layer.png"), "Layer",
                     "A layer with thickness and material");

    m_view->addGroup("Particle layouts");
    m_view->addEntry(ParticleLayoutItem::M_TYPE, icon("ParticleLayout.png"),
                     "Basic particle layout", "A layout of particles");

    m_view->addGroup("Interference functions");
    for (auto modelType : ItemCatalog::interferenceFunctionTypes()) {
        const auto ui = ItemCatalog::instance().uiInfo(modelType);
        m_view->addEntry(modelType, QIcon(ui.iconPath), ui.menuEntry, ui.description);
    }

    m_view->addGroup("Hard particles");
    for (auto modelType : ItemCatalog::formFactorHardParticleTypes()) {
        const auto ui = ItemCatalog::instance().uiInfo(modelType);
        m_view->addEntry("FormFactor" + modelType, QIcon(ui.iconPath), ui.menuEntry,
                         ui.description);
    }

    m_view->addGroup("Ripples");
    for (auto modelType : ItemCatalog::formFactorRippleTypes()) {
        const auto ui = ItemCatalog::instance().uiInfo(modelType);
        m_view->addEntry("FormFactor" + modelType, QIcon(ui.iconPath), ui.menuEntry,
                         ui.description);
    }

    m_view->addGroup("Transformations");
    m_view->addEntry(TransformationItem::M_TYPE, icon("Transformation.png"), "Rotation",
                     "Rotation applied to particles");

    m_view->addGroup("Particle assemblies");
    for (auto modelType : ItemCatalog::particleAssemblyTypes()) {
        const auto ui = ItemCatalog::instance().uiInfo(modelType);
        m_view->addEntry(modelType, QIcon(ui.iconPath), ui.menuEntry, ui.description);
    }

    m_view->addGroup("Demo samples");
    for (auto exampleName : GUIExamplesFactory::exampleNames()) {
        auto [title, description] = GUIExamplesFactory::exampleInfo(exampleName);
        m_view->addEntry(exampleName, QIcon(":/SampleDesignerToolbox/images/sample_layers2.png"),
                         title, description);
    }
}
