//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/SampleDesigner/SelectionContainerForm.h
//! @brief     Defines class SelectionContainerForm
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_SELECTIONCONTAINERFORM_H
#define BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_SELECTIONCONTAINERFORM_H

#include "GUI/Views/SampleDesigner/LayerEditorUtils.h"
#include "GUI/Views/SampleDesigner/SampleEditorController.h"
#include "GUI/utils/CustomEventFilters.h"
#include "GUI/utils/LayoutUtils.h"
#include <QComboBox>
#include <QGridLayout>

//! A widget to contain a selection, defined by a SelectionDescriptor.
//!
//! This SelectionContainerForm is limited to contain the selection combo box and a list of double
//! values represented by DoubleDescriptors. The list of DoubleDescriptors is queried by calling
//! LayerEditorUtils::doubleDescriptorsOfItem(). To have the correct DoubleDescriptors on this form,
//! you may have to overload this method according to your class and your needs. The overload will
//! expect the template type you defined for SelectionDescriptor.
//!
//! Example:
//! SelectionDescriptor<RotationItem*>  => LayerEditorUtils::doubleDescriptorsOfItem(RotationItem*)
//!
//! The connection from selection combo -> SelectionDescriptor is made via
//! SampleEditorController::setCurrentIndex(), where a command can be used for undo purposes.
//!
//! For each DoubleDescriptor, a unit-aware DoubleSpinBox is created. The connection from each
//! spinbox to the DoubleDescriptor is made via SampleEditorController::setDouble(), where a command
//! can be used for undo purposes.
//!
//! If a more complex selection shall be realized (e.g. with
//! sub-selections or different value types), this class is not sufficient.
class SelectionContainerForm : public QWidget {
public:
    template <typename T>
    SelectionContainerForm(QWidget* parent, const SelectionDescriptor<T>& d,
                           SampleEditorController* ec)
        : QWidget(parent), m_ec(ec)
    {
        if (d.currentItem != nullptr)
            currentValues = [=] {
                return LayerEditorUtils::doubleDescriptorsOfItem(d.currentItem());
            };
        else
            currentValues = [] { return DoubleDescriptors(); };

        m_gridLayout = new QGridLayout(this);
        m_gridLayout->setContentsMargins(0, 0, 0, 0);
        m_gridLayout->setSpacing(6);

        m_combo = new QComboBox;
        WheelEventEater::install(m_combo);
        m_combo->addItems(d.options);
        m_combo->setCurrentIndex(d.currentIndex());
        m_combo->setMaxVisibleItems(m_combo->count());

        QObject::connect(m_combo, QOverload<int>::of(&QComboBox::currentIndexChanged),
                         [=](int current) {
                             clear();
                             ec->setCurrentIndex(this, current, d);
                         });

        m_gridLayout->addWidget(m_combo, 1, 0);
        createDoubleEdits();
    }

    void createDoubleEdits()
    {
        LayerEditorUtils::addMultiPropertyToGrid(m_gridLayout, 1, currentValues(), m_ec, true);
    }

private:
    //! Remove all descriptors from the layout
    void clear()
    {
        auto layoutItemOfComboBox = m_gridLayout->itemAtPosition(1, 0);
        m_gridLayout->takeAt(m_gridLayout->indexOf(layoutItemOfComboBox));
        GUI::Utils::Layout::clearLayout(m_gridLayout, true);
        m_gridLayout->addWidget(layoutItemOfComboBox->widget(), 1, 0);
    }

private:
    QGridLayout* m_gridLayout;
    QComboBox* m_combo;
    function<DoubleDescriptors()> currentValues = nullptr;
    SampleEditorController* m_ec;
};


#endif // BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_SELECTIONCONTAINERFORM_H
