//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/mainwindow/LinkInstrumentManager.cpp
//! @brief     Implements class LinkInstrumentManager
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/mainwindow/LinkInstrumentManager.h"
#include "GUI/Items/InstrumentItems.h"
#include "GUI/Items/RealDataItem.h"
#include "GUI/Models/Data/RealDataModel.h"
#include "GUI/Models/Instrument/InstrumentModel.h"
#include "GUI/Views/ImportDataWidgets/ImportDataUtils.h"
#include "GUI/mainwindow/mainwindow.h"
#include <QMessageBox>
#include <QPushButton>

using namespace GUI::View;

namespace {
bool QuestionOnInstrumentReshaping(const QString& message)
{
    QMessageBox msgBox;
    msgBox.setText("Instrument description conflicts with the experimental data.");

    QString informative;
    informative.append(message);
    informative.append("\n\nDo you want to adjust the instrument to the experimental data?\n\n");
    msgBox.setInformativeText(informative);

    QPushButton* modifyInstrumentButton =
        msgBox.addButton("Yes, please modify instrument", QMessageBox::YesRole);
    msgBox.addButton("No, leave as it is", QMessageBox::NoRole);

    msgBox.exec();

    return msgBox.clickedButton() == modifyInstrumentButton;
}
} // namespace

LinkInstrumentManager::LinkInstrumentManager(InstrumentModel* instrumentModel,
                                             RealDataModel* realDataModel)
    : m_instrumentModel(instrumentModel), m_realDataModel(realDataModel)
{
    connect(m_instrumentModel, &InstrumentModel::instrumentAddedOrRemoved, this,
            &LinkInstrumentManager::onInstrumentAddedOrRemoved);

    connect(m_instrumentModel, &InstrumentModel::instrumentChanged, this,
            &LinkInstrumentManager::onInstrumentChanged);
}

bool LinkInstrumentManager::canLinkDataToInstrument(const RealDataItem* realDataItem,
                                                    const QString& identifier,
                                                    bool quiet /*= false*/)
{
    auto instrumentItem = m_instrumentModel->findInstrumentById(identifier);

    // linking to null instrument is possible, it means unlinking from currently linked
    if (!instrumentItem)
        return true;

    if (!ImportDataUtils::Compatible(*instrumentItem, *realDataItem)) {
        if (!quiet)
            QMessageBox::warning(baWin, "Can't link to instrument",
                                 "Can't link, data is incompatible with the instrument.");
        return false;
    }

    if (realDataItem->isSpecularData() && !realDataItem->hasNativeData()) {
        if (!quiet)
            QMessageBox::warning(baWin, "Can't link to instrument", "Can't link, data is empty.");
        return false;
    }

    if (instrumentItem->alignedWith(realDataItem))
        return true;

    QString message =
        realDataItem->holdsDimensionalData()
            ? "Experimental data carries information on the range/points of measurement."
            : ImportDataUtils::printShapeMessage(instrumentItem->shape(), realDataItem->shape());
    if (!QuestionOnInstrumentReshaping(message))
        return false;

    instrumentItem->updateToRealData(realDataItem);
    return true;
}

void LinkInstrumentManager::onInstrumentChanged(const InstrumentItem* instrument)
{
    // Run through all RealDataItem and refresh linking to match possible change in detector
    // axes definition.
    for (auto realDataItem : m_realDataModel->realDataItems())
        if (realDataItem->instrumentId() == instrument->id()) {
            if (!instrument->alignedWith(realDataItem))
                realDataItem->clearInstrumentId();
            else
                realDataItem->updateToInstrument(instrument);
        }
}

void LinkInstrumentManager::onInstrumentAddedOrRemoved()
{
    // remove links in realDataItems (in case of a linked instrument was removed)
    for (auto realDataItem : m_realDataModel->realDataItems())
        if (!m_instrumentModel->instrumentExists(realDataItem->instrumentId()))
            realDataItem->clearInstrumentId();
}
