//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Slice/Slice.cpp
//! @brief     Implements class Slice.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Resample/Slice/Slice.h"
#include "Resample/Slice/SliceStack.h"
#include "Sample/Interface/LayerRoughness.h"
#include "Sample/Material/MaterialUtils.h"

Slice::Slice(const ZLimits& zRange, const Material& material, const R3& B_field,
             const LayerRoughness* const roughness)
    : m_zRange(zRange), m_material(material), m_B_field(B_field), m_top_roughness(roughness)
{
}

Slice::Slice(const Slice& other)
    : Slice(other.m_zRange, other.m_material, other.m_B_field, other.m_top_roughness)
{
}

Slice::Slice(Slice&& other)
    : m_zRange{other.m_zRange}
    , m_material{std::move(other.m_material)}
    , m_B_field{other.m_B_field}
    , m_top_roughness{other.m_top_roughness}
{
}

Slice::~Slice() = default;

void Slice::setMaterial(const Material& material)
{
    m_material = material;
}

const Material& Slice::material() const
{
    return m_material;
}

double Slice::zBottom() const
{
    return m_zRange.zBottom();
}

double Slice::zTop() const
{
    return m_zRange.zTop();
}

double Slice::zTopOr0() const
{
    return m_zRange.zTopOr0();
}

double Slice::thicknessOr0() const
{
    return m_zRange.thicknessOr0();
}

const LayerRoughness* Slice::topRoughness() const
{
    return m_top_roughness;
}

complex_t Slice::scalarReducedPotential(R3 k, double n_ref) const
{
    complex_t n = m_material.refractiveIndex(2.0 * M_PI / k.mag());
    return MaterialUtils::ScalarReducedPotential(n, k, n_ref);
}

Eigen::Matrix2cd Slice::polarizedReducedPotential(R3 k, double n_ref) const
{
    complex_t n = m_material.refractiveIndex(2.0 * M_PI / k.mag());
    return MaterialUtils::PolarizedReducedPotential(n, m_B_field, k, n_ref);
}

void Slice::initBField(R3 h_field, double h_z)
{
    static constexpr double Magnetic_Permeability = 4e-7 * M_PI;
    m_B_field = Magnetic_Permeability * (h_field + m_material.magnetization());
    m_B_field.setZ(Magnetic_Permeability * h_z);
}

void Slice::invertBField()
{
    m_B_field = -m_B_field;
}
