//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Scattering/IBornFF.cpp
//! @brief     Implements interface class IBornFF.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "Sample/Scattering/IBornFF.h"
#include "Base/Utils/Algorithms.h"
#include "Base/Vector/WavevectorInfo.h"
#include "Sample/Scattering/Rotations.h"
#include "Sample/Shapes/IShape3D.h"
#include <stdexcept>

IBornFF::IBornFF() = default;

IBornFF::IBornFF(const NodeMeta& meta, const std::vector<double>& PValues)
    : IFormFactor(meta, PValues)
{
}

IBornFF::~IBornFF() = default;

complex_t IBornFF::theFF(const WavevectorInfo& wavevectors) const
{
    return evaluate_for_q(wavevectors.getQ());
}

Eigen::Matrix2cd IBornFF::thePolFF(const WavevectorInfo& wavevectors) const
{
    return evaluate_for_q_pol(wavevectors.getQ());
}

double IBornFF::bottomZ(const IRotation& rotation) const
{
    if (!m_shape3D)
        throw std::runtime_error("Bug: Form factor has no m_shape3D, cannot compute bottom z");
    return BottomZ(m_shape3D->vertices(), rotation);
}

double IBornFF::topZ(const IRotation& rotation) const
{
    if (!m_shape3D)
        throw std::runtime_error("Bug: Form factor has no m_shape3D, cannot compute top z");
    return TopZ(m_shape3D->vertices(), rotation);
}

bool IBornFF::canSliceAnalytically(const IRotation& rot) const
{
    if (rot.zInvariant())
        return true;
    return false;
}

Eigen::Matrix2cd IBornFF::evaluate_for_q_pol(C3 q) const
{
    return evaluate_for_q(q) * Eigen::Matrix2cd::Identity();
}

SlicingEffects IBornFF::computeSlicingEffects(ZLimits limits, const R3& position, double height)
{
    R3 new_position(position);
    double z_bottom = position.z();
    double z_top = position.z() + height;
    double dz_top = std::isinf(limits.zTop()) ? -1 : z_top - limits.zTop();
    double dz_bottom = std::isinf(limits.zBottom()) ? -1 : limits.zBottom() - z_bottom;
    ASSERT(dz_top >= 0 || dz_bottom >= 0);
    ASSERT(dz_bottom <= height);
    ASSERT(dz_top <= height);
    if (dz_bottom < 0)
        dz_bottom = 0;
    if (dz_top < 0)
        dz_top = 0;
    if (dz_bottom > 0)
        new_position.setZ(limits.zBottom());
    return {new_position, dz_bottom, dz_top};
}

double IBornFF::BottomZ(const std::vector<R3>& vertices, const IRotation& rotation)
{
    ASSERT(vertices.size());
    return BaseUtils::algo::min_value(
        vertices.begin(), vertices.end(),
        [&](const R3& vertex) -> double { return rotation.transformed(vertex).z(); });
}

double IBornFF::TopZ(const std::vector<R3>& vertices, const IRotation& rotation)
{
    ASSERT(vertices.size());
    return BaseUtils::algo::max_value(
        vertices.begin(), vertices.end(),
        [&](const R3& vertex) -> double { return rotation.transformed(vertex).z(); });
}
