#include "GUI/Items/InstrumentItems.h"
#include "GUI/Items/IntensityDataItem.h"
#include "GUI/Items/JobItemUtils.h"
#include "GUI/Items/RealDataItem.h"
#include "GUI/Models/Data/ApplicationModels.h"
#include "GUI/Models/Data/RealDataModel.h"
#include "GUI/Models/Instrument/InstrumentModel.h"
#include "GUI/Project/ProjectUtils.h"
#include "GUI/Project/projectdocument.h"
#include "GUI/utils/Helpers.h"
#include "Tests/GTestWrapper/google_test.h"
#include "Tests/Unit/GUI/Utils.h"
#include <QFileInfo>
#include <QSignalSpy>

class TestProjectDocument : public ::testing::Test {
protected:
    //! helper method to modify something in a model
    void modify_models(ProjectDocument& doc)
    {
        auto instrument = doc.instrumentModel()->instrumentItems().front();
        instrument->setId(GUI::Helpers::createUuid());
    }
};

TEST_F(TestProjectDocument, test_projectDocument)
{
    const QString projectDir("test_projectDocument");
    GuiUnittestUtils::create_dir(projectDir);
    const QString projectFileName(projectDir + "/document.pro");

    ProjectDocument document;
    // Checking initial document status
    EXPECT_FALSE(document.isModified());
    EXPECT_FALSE(document.hasValidNameAndPath());
    EXPECT_EQ(document.projectDir(), QString());
    EXPECT_EQ(document.projectName(), QString());
    EXPECT_EQ(document.projectFileName(), QString());

    auto instrument = document.instrumentModel()->insertItem<GISASInstrumentItem>();
    instrument->setItemName("GISAS");

    // Checking document name and isModified status after project save
    document.save(projectFileName);
    EXPECT_TRUE(document.hasValidNameAndPath());
    EXPECT_EQ(document.projectDir(), projectDir);
    EXPECT_EQ(document.projectName(), "document");
    EXPECT_EQ(document.projectFileName(), projectFileName);
    EXPECT_FALSE(document.isModified());

    QSignalSpy spyDocument(&document, SIGNAL(modified()));
    EXPECT_EQ(spyDocument.count(), 0);

    // Changing document and checking its status
    modify_models(document);
    EXPECT_TRUE(document.isModified());
    EXPECT_EQ(spyDocument.count(), 1);

    // Saving document
    document.save(projectFileName);
    EXPECT_FALSE(document.isModified());
    EXPECT_EQ(spyDocument.count(), 2); // save triggers 'modified' signal

    QFileInfo info(projectFileName);
    EXPECT_TRUE(info.exists());
}

TEST_F(TestProjectDocument, test_projectDocumentWithData)
{
    const QString projectDir("test_projectDocumentWithData");
    GuiUnittestUtils::create_dir(projectDir);

    ProjectDocument document;
    auto instrument = document.instrumentModel()->insertItem<GISASInstrumentItem>();
    instrument->setItemName("GISAS");
    RealDataItem* realData =
        GuiUnittestUtils::createRealData("RealData", *document.realDataModel());
    ASSERT(realData);
    DataItem* intensityItem = realData->dataItem();
    GUI::Model::JobItemUtils::createDefaultDetectorMap(
        intensityItem, document.instrumentModel()->instrument2DItems().front());
    intensityItem->setFileName("realdata.int.gz");

    document.save(projectDir + "/untitled.pro");

    QFileInfo info(projectDir + "/untitled.pro");
    EXPECT_TRUE(info.exists());

    info.setFile(projectDir + "/realdata.int.gz");
    EXPECT_TRUE(info.exists());
}
